# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Optional

import httpx

from .type import (
    TypeResource,
    AsyncTypeResource,
    TypeResourceWithRawResponse,
    AsyncTypeResourceWithRawResponse,
    TypeResourceWithStreamingResponse,
    AsyncTypeResourceWithStreamingResponse,
)
from .class_ import (
    ClassResource,
    AsyncClassResource,
    ClassResourceWithRawResponse,
    AsyncClassResourceWithRawResponse,
    ClassResourceWithStreamingResponse,
    AsyncClassResourceWithStreamingResponse,
)
from .context import (
    ContextResource,
    AsyncContextResource,
    ContextResourceWithRawResponse,
    AsyncContextResourceWithRawResponse,
    ContextResourceWithStreamingResponse,
    AsyncContextResourceWithStreamingResponse,
)
from .severity import (
    SeverityResource,
    AsyncSeverityResource,
    SeverityResourceWithRawResponse,
    AsyncSeverityResourceWithRawResponse,
    SeverityResourceWithStreamingResponse,
    AsyncSeverityResourceWithStreamingResponse,
)
from ...._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from .audit_logs import (
    AuditLogsResource,
    AsyncAuditLogsResource,
    AuditLogsResourceWithRawResponse,
    AsyncAuditLogsResourceWithRawResponse,
    AuditLogsResourceWithStreamingResponse,
    AsyncAuditLogsResourceWithStreamingResponse,
)
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....pagination import SyncV4PagePagination, AsyncV4PagePagination
from .classification import (
    ClassificationResource,
    AsyncClassificationResource,
    ClassificationResourceWithRawResponse,
    AsyncClassificationResourceWithRawResponse,
    ClassificationResourceWithStreamingResponse,
    AsyncClassificationResourceWithStreamingResponse,
)
from ...._base_client import AsyncPaginator, make_request_options
from ....types.security_center import insight_list_params, insight_dismiss_params
from ....types.security_center.insight_list_response import InsightListResponse
from ....types.intel.attack_surface_report.issue_type import IssueType
from ....types.security_center.insight_dismiss_response import InsightDismissResponse
from ....types.intel.attack_surface_report.severity_query_param import SeverityQueryParam

__all__ = ["InsightsResource", "AsyncInsightsResource"]


class InsightsResource(SyncAPIResource):
    @cached_property
    def class_(self) -> ClassResource:
        return ClassResource(self._client)

    @cached_property
    def severity(self) -> SeverityResource:
        return SeverityResource(self._client)

    @cached_property
    def type(self) -> TypeResource:
        return TypeResource(self._client)

    @cached_property
    def audit_logs(self) -> AuditLogsResource:
        return AuditLogsResource(self._client)

    @cached_property
    def classification(self) -> ClassificationResource:
        return ClassificationResource(self._client)

    @cached_property
    def context(self) -> ContextResource:
        return ContextResource(self._client)

    @cached_property
    def with_raw_response(self) -> InsightsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return InsightsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> InsightsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return InsightsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        dismissed: bool | Omit = omit,
        issue_class: SequenceNotStr[str] | Omit = omit,
        issue_class_neq: SequenceNotStr[str] | Omit = omit,
        issue_type: List[IssueType] | Omit = omit,
        issue_type_neq: List[IssueType] | Omit = omit,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        product: SequenceNotStr[str] | Omit = omit,
        product_neq: SequenceNotStr[str] | Omit = omit,
        severity: List[SeverityQueryParam] | Omit = omit,
        severity_neq: List[SeverityQueryParam] | Omit = omit,
        subject: SequenceNotStr[str] | Omit = omit,
        subject_neq: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePagination[Optional[InsightListResponse]]:
        """
        Lists all Security Center insights for the account or zone, showing security
        findings and recommendations.

        Args:
          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          page: Specifies the current page within paginated list of results.

          per_page: Sets the number of results per page of results.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return self._get_api_list(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/security-center/insights",
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            page=SyncV4PagePagination[Optional[InsightListResponse]],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "dismissed": dismissed,
                        "issue_class": issue_class,
                        "issue_class_neq": issue_class_neq,
                        "issue_type": issue_type,
                        "issue_type_neq": issue_type_neq,
                        "page": page,
                        "per_page": per_page,
                        "product": product,
                        "product_neq": product_neq,
                        "severity": severity,
                        "severity_neq": severity_neq,
                        "subject": subject,
                        "subject_neq": subject_neq,
                    },
                    insight_list_params.InsightListParams,
                ),
            ),
            model=InsightListResponse,
        )

    def dismiss(
        self,
        issue_id: str,
        *,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        dismiss: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InsightDismissResponse:
        """
        Archives a Security Center insight for an account or zone, removing it from the
        active insights list while preserving historical data.

        Args:
          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not issue_id:
            raise ValueError(f"Expected a non-empty value for `issue_id` but received {issue_id!r}")
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return self._put(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/security-center/insights/{issue_id}/dismiss",
                issue_id=issue_id,
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            body=maybe_transform({"dismiss": dismiss}, insight_dismiss_params.InsightDismissParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=InsightDismissResponse,
        )


class AsyncInsightsResource(AsyncAPIResource):
    @cached_property
    def class_(self) -> AsyncClassResource:
        return AsyncClassResource(self._client)

    @cached_property
    def severity(self) -> AsyncSeverityResource:
        return AsyncSeverityResource(self._client)

    @cached_property
    def type(self) -> AsyncTypeResource:
        return AsyncTypeResource(self._client)

    @cached_property
    def audit_logs(self) -> AsyncAuditLogsResource:
        return AsyncAuditLogsResource(self._client)

    @cached_property
    def classification(self) -> AsyncClassificationResource:
        return AsyncClassificationResource(self._client)

    @cached_property
    def context(self) -> AsyncContextResource:
        return AsyncContextResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncInsightsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncInsightsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncInsightsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncInsightsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        dismissed: bool | Omit = omit,
        issue_class: SequenceNotStr[str] | Omit = omit,
        issue_class_neq: SequenceNotStr[str] | Omit = omit,
        issue_type: List[IssueType] | Omit = omit,
        issue_type_neq: List[IssueType] | Omit = omit,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        product: SequenceNotStr[str] | Omit = omit,
        product_neq: SequenceNotStr[str] | Omit = omit,
        severity: List[SeverityQueryParam] | Omit = omit,
        severity_neq: List[SeverityQueryParam] | Omit = omit,
        subject: SequenceNotStr[str] | Omit = omit,
        subject_neq: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Optional[InsightListResponse], AsyncV4PagePagination[Optional[InsightListResponse]]]:
        """
        Lists all Security Center insights for the account or zone, showing security
        findings and recommendations.

        Args:
          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          page: Specifies the current page within paginated list of results.

          per_page: Sets the number of results per page of results.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return self._get_api_list(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/security-center/insights",
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            page=AsyncV4PagePagination[Optional[InsightListResponse]],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "dismissed": dismissed,
                        "issue_class": issue_class,
                        "issue_class_neq": issue_class_neq,
                        "issue_type": issue_type,
                        "issue_type_neq": issue_type_neq,
                        "page": page,
                        "per_page": per_page,
                        "product": product,
                        "product_neq": product_neq,
                        "severity": severity,
                        "severity_neq": severity_neq,
                        "subject": subject,
                        "subject_neq": subject_neq,
                    },
                    insight_list_params.InsightListParams,
                ),
            ),
            model=InsightListResponse,
        )

    async def dismiss(
        self,
        issue_id: str,
        *,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        dismiss: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InsightDismissResponse:
        """
        Archives a Security Center insight for an account or zone, removing it from the
        active insights list while preserving historical data.

        Args:
          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not issue_id:
            raise ValueError(f"Expected a non-empty value for `issue_id` but received {issue_id!r}")
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return await self._put(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/security-center/insights/{issue_id}/dismiss",
                issue_id=issue_id,
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            body=await async_maybe_transform({"dismiss": dismiss}, insight_dismiss_params.InsightDismissParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=InsightDismissResponse,
        )


class InsightsResourceWithRawResponse:
    def __init__(self, insights: InsightsResource) -> None:
        self._insights = insights

        self.list = to_raw_response_wrapper(
            insights.list,
        )
        self.dismiss = to_raw_response_wrapper(
            insights.dismiss,
        )

    @cached_property
    def class_(self) -> ClassResourceWithRawResponse:
        return ClassResourceWithRawResponse(self._insights.class_)

    @cached_property
    def severity(self) -> SeverityResourceWithRawResponse:
        return SeverityResourceWithRawResponse(self._insights.severity)

    @cached_property
    def type(self) -> TypeResourceWithRawResponse:
        return TypeResourceWithRawResponse(self._insights.type)

    @cached_property
    def audit_logs(self) -> AuditLogsResourceWithRawResponse:
        return AuditLogsResourceWithRawResponse(self._insights.audit_logs)

    @cached_property
    def classification(self) -> ClassificationResourceWithRawResponse:
        return ClassificationResourceWithRawResponse(self._insights.classification)

    @cached_property
    def context(self) -> ContextResourceWithRawResponse:
        return ContextResourceWithRawResponse(self._insights.context)


class AsyncInsightsResourceWithRawResponse:
    def __init__(self, insights: AsyncInsightsResource) -> None:
        self._insights = insights

        self.list = async_to_raw_response_wrapper(
            insights.list,
        )
        self.dismiss = async_to_raw_response_wrapper(
            insights.dismiss,
        )

    @cached_property
    def class_(self) -> AsyncClassResourceWithRawResponse:
        return AsyncClassResourceWithRawResponse(self._insights.class_)

    @cached_property
    def severity(self) -> AsyncSeverityResourceWithRawResponse:
        return AsyncSeverityResourceWithRawResponse(self._insights.severity)

    @cached_property
    def type(self) -> AsyncTypeResourceWithRawResponse:
        return AsyncTypeResourceWithRawResponse(self._insights.type)

    @cached_property
    def audit_logs(self) -> AsyncAuditLogsResourceWithRawResponse:
        return AsyncAuditLogsResourceWithRawResponse(self._insights.audit_logs)

    @cached_property
    def classification(self) -> AsyncClassificationResourceWithRawResponse:
        return AsyncClassificationResourceWithRawResponse(self._insights.classification)

    @cached_property
    def context(self) -> AsyncContextResourceWithRawResponse:
        return AsyncContextResourceWithRawResponse(self._insights.context)


class InsightsResourceWithStreamingResponse:
    def __init__(self, insights: InsightsResource) -> None:
        self._insights = insights

        self.list = to_streamed_response_wrapper(
            insights.list,
        )
        self.dismiss = to_streamed_response_wrapper(
            insights.dismiss,
        )

    @cached_property
    def class_(self) -> ClassResourceWithStreamingResponse:
        return ClassResourceWithStreamingResponse(self._insights.class_)

    @cached_property
    def severity(self) -> SeverityResourceWithStreamingResponse:
        return SeverityResourceWithStreamingResponse(self._insights.severity)

    @cached_property
    def type(self) -> TypeResourceWithStreamingResponse:
        return TypeResourceWithStreamingResponse(self._insights.type)

    @cached_property
    def audit_logs(self) -> AuditLogsResourceWithStreamingResponse:
        return AuditLogsResourceWithStreamingResponse(self._insights.audit_logs)

    @cached_property
    def classification(self) -> ClassificationResourceWithStreamingResponse:
        return ClassificationResourceWithStreamingResponse(self._insights.classification)

    @cached_property
    def context(self) -> ContextResourceWithStreamingResponse:
        return ContextResourceWithStreamingResponse(self._insights.context)


class AsyncInsightsResourceWithStreamingResponse:
    def __init__(self, insights: AsyncInsightsResource) -> None:
        self._insights = insights

        self.list = async_to_streamed_response_wrapper(
            insights.list,
        )
        self.dismiss = async_to_streamed_response_wrapper(
            insights.dismiss,
        )

    @cached_property
    def class_(self) -> AsyncClassResourceWithStreamingResponse:
        return AsyncClassResourceWithStreamingResponse(self._insights.class_)

    @cached_property
    def severity(self) -> AsyncSeverityResourceWithStreamingResponse:
        return AsyncSeverityResourceWithStreamingResponse(self._insights.severity)

    @cached_property
    def type(self) -> AsyncTypeResourceWithStreamingResponse:
        return AsyncTypeResourceWithStreamingResponse(self._insights.type)

    @cached_property
    def audit_logs(self) -> AsyncAuditLogsResourceWithStreamingResponse:
        return AsyncAuditLogsResourceWithStreamingResponse(self._insights.audit_logs)

    @cached_property
    def classification(self) -> AsyncClassificationResourceWithStreamingResponse:
        return AsyncClassificationResourceWithStreamingResponse(self._insights.classification)

    @cached_property
    def context(self) -> AsyncContextResourceWithStreamingResponse:
        return AsyncContextResourceWithStreamingResponse(self._insights.context)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ...._models import BaseModel

__all__ = ["HealthGetResponse", "POPHealth", "POPHealthOrigin", "POPHealthOriginIP"]


class POPHealthOriginIP(BaseModel):
    failure_reason: Optional[str] = None
    """Failure reason."""

    healthy: Optional[bool] = None
    """Origin health status."""

    response_code: Optional[float] = None
    """Response code from origin health check."""

    rtt: Optional[str] = None
    """Origin RTT (Round Trip Time) response."""


class POPHealthOrigin(BaseModel):
    ip: Optional[POPHealthOriginIP] = None


class POPHealth(BaseModel):
    """List of regions and associated health status."""

    healthy: Optional[bool] = None
    """Whether health check in region is healthy."""

    origins: Optional[List[POPHealthOrigin]] = None


class HealthGetResponse(BaseModel):
    """A list of regions from which to run health checks.

    Null means every Cloudflare data center.
    """

    pool_id: Optional[str] = None
    """Pool ID."""

    pop_health: Optional[POPHealth] = None
    """List of regions and associated health status."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["NeoEvent"]


class NeoEvent(BaseModel):
    action_type: str = FieldInfo(alias="actionType")
    """Tool event type."""

    blocking_triggers: List[str] = FieldInfo(alias="blockingTriggers")
    """List of blocking triggers IDs."""

    data: object
    """Event payload."""

    firing_triggers: List[str] = FieldInfo(alias="firingTriggers")
    """List of firing triggers IDs."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .deployment import Deployment as Deployment
from .consumer_script import ConsumerScript as ConsumerScript
from .secret_get_params import SecretGetParams as SecretGetParams
from .tail_get_response import TailGetResponse as TailGetResponse
from .tail_create_params import TailCreateParams as TailCreateParams
from .secret_get_response import SecretGetResponse as SecretGetResponse
from .setting_edit_params import SettingEditParams as SettingEditParams
from .version_list_params import VersionListParams as VersionListParams
from .secret_delete_params import SecretDeleteParams as SecretDeleteParams
from .secret_list_response import SecretListResponse as SecretListResponse
from .secret_update_params import SecretUpdateParams as SecretUpdateParams
from .tail_create_response import TailCreateResponse as TailCreateResponse
from .tail_delete_response import TailDeleteResponse as TailDeleteResponse
from .version_get_response import VersionGetResponse as VersionGetResponse
from .consumer_script_param import ConsumerScriptParam as ConsumerScriptParam
from .content_update_params import ContentUpdateParams as ContentUpdateParams
from .schedule_get_response import ScheduleGetResponse as ScheduleGetResponse
from .version_create_params import VersionCreateParams as VersionCreateParams
from .version_list_response import VersionListResponse as VersionListResponse
from .schedule_update_params import ScheduleUpdateParams as ScheduleUpdateParams
from .secret_update_response import SecretUpdateResponse as SecretUpdateResponse
from .subdomain_get_response import SubdomainGetResponse as SubdomainGetResponse
from .subdomain_create_params import SubdomainCreateParams as SubdomainCreateParams
from .version_create_response import VersionCreateResponse as VersionCreateResponse
from .deployment_create_params import DeploymentCreateParams as DeploymentCreateParams
from .deployment_list_response import DeploymentListResponse as DeploymentListResponse
from .schedule_update_response import ScheduleUpdateResponse as ScheduleUpdateResponse
from .secret_bulk_update_params import SecretBulkUpdateParams as SecretBulkUpdateParams
from .subdomain_create_response import SubdomainCreateResponse as SubdomainCreateResponse
from .subdomain_delete_response import SubdomainDeleteResponse as SubdomainDeleteResponse
from .deployment_delete_response import DeploymentDeleteResponse as DeploymentDeleteResponse
from .secret_bulk_update_response import SecretBulkUpdateResponse as SecretBulkUpdateResponse
from .script_and_version_setting_edit_params import (
    ScriptAndVersionSettingEditParams as ScriptAndVersionSettingEditParams,
)
from .script_and_version_setting_get_response import (
    ScriptAndVersionSettingGetResponse as ScriptAndVersionSettingGetResponse,
)
from .script_and_version_setting_edit_response import (
    ScriptAndVersionSettingEditResponse as ScriptAndVersionSettingEditResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Union, Optional
from typing_extensions import Literal, TypeAlias

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = [
    "TelemetryQueryResponse",
    "Run",
    "RunQuery",
    "RunQueryParameters",
    "RunQueryParametersCalculation",
    "RunQueryParametersFilter",
    "RunQueryParametersFilterUnionMember0",
    "RunQueryParametersFilterWorkersObservabilityFilterLeaf",
    "RunQueryParametersGroupBy",
    "RunQueryParametersHaving",
    "RunQueryParametersNeedle",
    "RunQueryParametersNeedleValue",
    "RunQueryParametersOrderBy",
    "RunTimeframe",
    "RunStatistics",
    "Statistics",
    "Agent",
    "Calculation",
    "CalculationAggregate",
    "CalculationAggregateGroup",
    "CalculationSeries",
    "CalculationSeriesData",
    "CalculationSeriesDataGroup",
    "Compare",
    "CompareAggregate",
    "CompareAggregateGroup",
    "CompareSeries",
    "CompareSeriesData",
    "CompareSeriesDataGroup",
    "Events",
    "EventsEvent",
    "EventsEventMetadata",
    "EventsEventWorkers",
    "EventsEventWorkersUnionMember0",
    "EventsEventWorkersUnionMember0ScriptVersion",
    "EventsEventWorkersUnionMember1",
    "EventsEventWorkersUnionMember1DiagnosticsChannelEvent",
    "EventsEventWorkersUnionMember1ScriptVersion",
    "EventsField",
    "EventsSeries",
    "EventsSeriesData",
    "EventsSeriesDataAggregates",
    "Invocation",
    "InvocationMetadata",
    "InvocationWorkers",
    "InvocationWorkersUnionMember0",
    "InvocationWorkersUnionMember0ScriptVersion",
    "InvocationWorkersUnionMember1",
    "InvocationWorkersUnionMember1DiagnosticsChannelEvent",
    "InvocationWorkersUnionMember1ScriptVersion",
    "Trace",
]


class RunQueryParametersCalculation(BaseModel):
    operator: Literal[
        "uniq",
        "count",
        "max",
        "min",
        "sum",
        "avg",
        "median",
        "p001",
        "p01",
        "p05",
        "p10",
        "p25",
        "p75",
        "p90",
        "p95",
        "p99",
        "p999",
        "stddev",
        "variance",
        "COUNT_DISTINCT",
        "COUNT",
        "MAX",
        "MIN",
        "SUM",
        "AVG",
        "MEDIAN",
        "P001",
        "P01",
        "P05",
        "P10",
        "P25",
        "P75",
        "P90",
        "P95",
        "P99",
        "P999",
        "STDDEV",
        "VARIANCE",
    ]

    alias: Optional[str] = None

    key: Optional[str] = None

    key_type: Optional[Literal["string", "number", "boolean"]] = FieldInfo(alias="keyType", default=None)


class RunQueryParametersFilterUnionMember0(BaseModel):
    filter_combination: Literal["and", "or", "AND", "OR"] = FieldInfo(alias="filterCombination")

    filters: List[object]

    kind: Literal["group"]


class RunQueryParametersFilterWorkersObservabilityFilterLeaf(BaseModel):
    """A filter condition applied to query results.

    Use the keys and values endpoints to discover available fields and their values before constructing filters.
    """

    key: str
    """Filter field name.

    Use verified keys from previous query results or the keys endpoint. Common keys
    include $metadata.service, $metadata.origin, $metadata.trigger,
    $metadata.message, and $metadata.error.
    """

    operation: Literal[
        "includes",
        "not_includes",
        "starts_with",
        "regex",
        "exists",
        "is_null",
        "in",
        "not_in",
        "eq",
        "neq",
        "gt",
        "gte",
        "lt",
        "lte",
        "=",
        "!=",
        ">",
        ">=",
        "<",
        "<=",
        "INCLUDES",
        "DOES_NOT_INCLUDE",
        "MATCH_REGEX",
        "EXISTS",
        "DOES_NOT_EXIST",
        "IN",
        "NOT_IN",
        "STARTS_WITH",
    ]
    """Comparison operator.

    String operators: includes, not_includes, starts_with, regex. Existence: exists,
    is_null. Set membership: in, not_in (comma-separated values). Numeric: eq, neq,
    gt, gte, lt, lte.
    """

    type: Literal["string", "number", "boolean"]
    """Data type of the filter field.

    Must match the actual type of the key being filtered.
    """

    kind: Optional[Literal["filter"]] = None
    """Discriminator for leaf filter nodes.

    Always 'filter' when present; may be omitted.
    """

    value: Union[str, float, bool, None] = None
    """Comparison value.

    Must match actual values in your data — verify with the values endpoint. Ensure
    the value type (string/number/boolean) matches the field type. String
    comparisons are case-sensitive. Regex uses RE2 syntax (no
    lookaheads/lookbehinds).
    """


RunQueryParametersFilter: TypeAlias = Union[
    RunQueryParametersFilterUnionMember0, RunQueryParametersFilterWorkersObservabilityFilterLeaf
]


class RunQueryParametersGroupBy(BaseModel):
    type: Literal["string", "number", "boolean"]

    value: str


class RunQueryParametersHaving(BaseModel):
    key: str

    operation: Literal["eq", "neq", "gt", "gte", "lt", "lte"]

    value: float


class RunQueryParametersNeedleValue(BaseModel):
    model_config = {"arbitrary_types_allowed": True}


class RunQueryParametersNeedle(BaseModel):
    """Define an expression to search using full-text search."""

    value: RunQueryParametersNeedleValue

    is_regex: Optional[bool] = FieldInfo(alias="isRegex", default=None)

    match_case: Optional[bool] = FieldInfo(alias="matchCase", default=None)


class RunQueryParametersOrderBy(BaseModel):
    """Configure the order of the results returned by the query."""

    value: str
    """Configure which Calculation to order the results by."""

    order: Optional[Literal["asc", "desc"]] = None
    """Set the order of the results"""


class RunQueryParameters(BaseModel):
    calculations: Optional[List[RunQueryParametersCalculation]] = None
    """Create Calculations to compute as part of the query."""

    datasets: Optional[List[str]] = None
    """Set the Datasets to query. Leave it empty to query all the datasets."""

    filter_combination: Optional[Literal["and", "or", "AND", "OR"]] = FieldInfo(alias="filterCombination", default=None)
    """Set a Flag to describe how to combine the filters on the query."""

    filters: Optional[List[RunQueryParametersFilter]] = None
    """Configure the Filters to apply to the query.

    Supports nested groups via kind: 'group'.
    """

    group_bys: Optional[List[RunQueryParametersGroupBy]] = FieldInfo(alias="groupBys", default=None)
    """Define how to group the results of the query."""

    havings: Optional[List[RunQueryParametersHaving]] = None
    """Configure the Having clauses that filter on calculations in the query result."""

    limit: Optional[int] = None
    """Set a limit on the number of results / records returned by the query"""

    needle: Optional[RunQueryParametersNeedle] = None
    """Define an expression to search using full-text search."""

    order_by: Optional[RunQueryParametersOrderBy] = FieldInfo(alias="orderBy", default=None)
    """Configure the order of the results returned by the query."""


class RunQuery(BaseModel):
    """
    A saved query definition with its parameters, metadata, and ownership information.
    """

    id: str

    adhoc: bool
    """If the query wasn't explcitly saved"""

    created: str

    created_by: str = FieldInfo(alias="createdBy")

    description: Optional[str] = None

    name: str
    """Query name"""

    parameters: RunQueryParameters

    updated: str

    updated_by: str = FieldInfo(alias="updatedBy")


class RunTimeframe(BaseModel):
    """Time range for the query execution"""

    from_: float = FieldInfo(alias="from")
    """Start timestamp for the query timeframe (Unix timestamp in milliseconds)"""

    to: float
    """End timestamp for the query timeframe (Unix timestamp in milliseconds)"""


class RunStatistics(BaseModel):
    """
    Query performance statistics from the database (does not include network latency).
    """

    bytes_read: float
    """Number of uncompressed bytes read from the table."""

    elapsed: float
    """Time in seconds for the query to run."""

    rows_read: float
    """Number of rows scanned from the table."""

    abr_level: Optional[float] = None
    """The level of Adaptive Bit Rate (ABR) sampling used for the query.

    If empty the ABR level is 1
    """


class Run(BaseModel):
    """
    The query run metadata including the query definition, execution status, and timeframe.
    """

    id: str
    """Unique identifier for this query run."""

    account_id: str = FieldInfo(alias="accountId")
    """Cloudflare account ID that owns this query run."""

    dry: bool
    """Whether this was a dry run (results not persisted)."""

    granularity: float
    """Number of time-series buckets used for the query.

    Higher values produce more detailed series data.
    """

    query: RunQuery
    """
    A saved query definition with its parameters, metadata, and ownership
    information.
    """

    status: Literal["STARTED", "COMPLETED"]
    """Current execution status of the query run."""

    timeframe: RunTimeframe
    """Time range for the query execution"""

    user_id: str = FieldInfo(alias="userId")
    """ID of the user who initiated the query run."""

    created: Optional[str] = None
    """ISO-8601 timestamp when the query run was created."""

    statistics: Optional[RunStatistics] = None
    """
    Query performance statistics from the database (does not include network
    latency).
    """

    updated: Optional[str] = None
    """ISO-8601 timestamp when the query run was last updated."""


class Statistics(BaseModel):
    """Query performance statistics from the database.

    Includes execution time, rows scanned, and bytes read. Does not include network latency.
    """

    bytes_read: float
    """Number of uncompressed bytes read from the table."""

    elapsed: float
    """Time in seconds for the query to run."""

    rows_read: float
    """Number of rows scanned from the table."""

    abr_level: Optional[float] = None
    """The level of Adaptive Bit Rate (ABR) sampling used for the query.

    If empty the ABR level is 1
    """


class Agent(BaseModel):
    agent_class: str = FieldInfo(alias="agentClass")
    """Class name of the Durable Object agent."""

    event_type_counts: Dict[str, float] = FieldInfo(alias="eventTypeCounts")
    """Breakdown of event counts by event type."""

    first_event_ms: float = FieldInfo(alias="firstEventMs")
    """
    Timestamp of the earliest event from this agent in the queried window (Unix
    epoch ms).
    """

    has_errors: bool = FieldInfo(alias="hasErrors")
    """Whether the agent emitted any error events in the queried window."""

    last_event_ms: float = FieldInfo(alias="lastEventMs")
    """Timestamp of the most recent event from this agent (Unix epoch ms)."""

    namespace: str
    """Durable Object namespace the agent belongs to."""

    service: str
    """Worker service name that hosts this agent."""

    total_events: float = FieldInfo(alias="totalEvents")
    """Total number of events emitted by this agent in the queried window."""


class CalculationAggregateGroup(BaseModel):
    key: str

    value: Union[str, float, bool]


class CalculationAggregate(BaseModel):
    count: float

    interval: float

    sample_interval: float = FieldInfo(alias="sampleInterval")

    value: float

    groups: Optional[List[CalculationAggregateGroup]] = None


class CalculationSeriesDataGroup(BaseModel):
    key: str

    value: Union[str, float, bool]


class CalculationSeriesData(BaseModel):
    count: float

    interval: float

    sample_interval: float = FieldInfo(alias="sampleInterval")

    value: float

    first_seen: Optional[str] = FieldInfo(alias="firstSeen", default=None)

    groups: Optional[List[CalculationSeriesDataGroup]] = None

    last_seen: Optional[str] = FieldInfo(alias="lastSeen", default=None)


class CalculationSeries(BaseModel):
    data: List[CalculationSeriesData]

    time: str


class Calculation(BaseModel):
    aggregates: List[CalculationAggregate]

    calculation: str

    series: List[CalculationSeries]

    alias: Optional[str] = None


class CompareAggregateGroup(BaseModel):
    key: str

    value: Union[str, float, bool]


class CompareAggregate(BaseModel):
    count: float

    interval: float

    sample_interval: float = FieldInfo(alias="sampleInterval")

    value: float

    groups: Optional[List[CompareAggregateGroup]] = None


class CompareSeriesDataGroup(BaseModel):
    key: str

    value: Union[str, float, bool]


class CompareSeriesData(BaseModel):
    count: float

    interval: float

    sample_interval: float = FieldInfo(alias="sampleInterval")

    value: float

    first_seen: Optional[str] = FieldInfo(alias="firstSeen", default=None)

    groups: Optional[List[CompareSeriesDataGroup]] = None

    last_seen: Optional[str] = FieldInfo(alias="lastSeen", default=None)


class CompareSeries(BaseModel):
    data: List[CompareSeriesData]

    time: str


class Compare(BaseModel):
    aggregates: List[CompareAggregate]

    calculation: str

    series: List[CompareSeries]

    alias: Optional[str] = None


class EventsEventMetadata(BaseModel):
    """Structured metadata extracted from the event.

    These fields are indexed and available for filtering and aggregation.
    """

    id: str
    """Unique event ID. Use as the cursor value for offset-based pagination."""

    account: Optional[str] = None
    """Cloudflare account identifier."""

    cloud_service: Optional[str] = FieldInfo(alias="cloudService", default=None)
    """Cloudflare product that generated this event (e.g. workers, pages)."""

    cold_start: Optional[int] = FieldInfo(alias="coldStart", default=None)
    """Whether this was a cold start (1) or warm invocation (0)."""

    cost: Optional[int] = None
    """Estimated cost units for this invocation."""

    duration: Optional[int] = None
    """Span duration in milliseconds."""

    end_time: Optional[int] = FieldInfo(alias="endTime", default=None)
    """Span end time as a Unix epoch in milliseconds."""

    error: Optional[str] = None
    """Error message, present when the log represents an error."""

    error_template: Optional[str] = FieldInfo(alias="errorTemplate", default=None)
    """Templatized version of the error message used for grouping similar errors."""

    fingerprint: Optional[str] = None
    """Content-based fingerprint used to group similar events."""

    level: Optional[str] = None
    """Log level (e.g. log, debug, info, warn, error)."""

    message: Optional[str] = None
    """Log message text."""

    message_template: Optional[str] = FieldInfo(alias="messageTemplate", default=None)
    """Templatized version of the log message used for grouping similar messages."""

    metric_name: Optional[str] = FieldInfo(alias="metricName", default=None)
    """Metric name when the event represents a metric data point."""

    origin: Optional[str] = None
    """Origin of the event (e.g. fetch, scheduled, queue)."""

    parent_span_id: Optional[str] = FieldInfo(alias="parentSpanId", default=None)
    """Span ID of the parent span in the trace hierarchy."""

    provider: Optional[str] = None
    """Infrastructure provider identifier."""

    region: Optional[str] = None
    """Cloudflare data center / region that handled the request."""

    request_id: Optional[str] = FieldInfo(alias="requestId", default=None)
    """Cloudflare request ID that ties all logs from a single invocation together."""

    service: Optional[str] = None
    """Worker script name that produced this event."""

    span_id: Optional[str] = FieldInfo(alias="spanId", default=None)
    """Span ID for this individual unit of work within a trace."""

    span_name: Optional[str] = FieldInfo(alias="spanName", default=None)
    """Human-readable name for this span."""

    stack_id: Optional[str] = FieldInfo(alias="stackId", default=None)
    """Stack / deployment identifier."""

    start_time: Optional[int] = FieldInfo(alias="startTime", default=None)
    """Span start time as a Unix epoch in milliseconds."""

    status_code: Optional[int] = FieldInfo(alias="statusCode", default=None)
    """HTTP response status code returned by the Worker."""

    trace_duration: Optional[int] = FieldInfo(alias="traceDuration", default=None)
    """Total duration of the entire trace in milliseconds."""

    trace_id: Optional[str] = FieldInfo(alias="traceId", default=None)
    """Distributed trace ID linking spans across services."""

    transaction_name: Optional[str] = FieldInfo(alias="transactionName", default=None)
    """Logical transaction name for this request."""

    trigger: Optional[str] = None
    """What triggered the invocation (e.g. GET /users, POST /orders, queue message)."""

    type: Optional[str] = None
    """Event type classifier (e.g. cf-worker-event, cf-worker-log)."""

    url: Optional[str] = None
    """Request URL that triggered the Worker invocation."""


class EventsEventWorkersUnionMember0ScriptVersion(BaseModel):
    id: Optional[str] = None

    message: Optional[str] = None

    tag: Optional[str] = None


class EventsEventWorkersUnionMember0(BaseModel):
    event_type: Literal[
        "fetch", "scheduled", "alarm", "cron", "queue", "email", "tail", "rpc", "websocket", "workflow", "unknown"
    ] = FieldInfo(alias="eventType")

    request_id: str = FieldInfo(alias="requestId")

    script_name: str = FieldInfo(alias="scriptName")

    durable_object_id: Optional[str] = FieldInfo(alias="durableObjectId", default=None)

    entrypoint: Optional[str] = None

    event: Optional[Dict[str, object]] = None

    execution_model: Optional[Literal["durableObject", "stateless"]] = FieldInfo(alias="executionModel", default=None)

    outcome: Optional[str] = None

    script_version: Optional[EventsEventWorkersUnionMember0ScriptVersion] = FieldInfo(
        alias="scriptVersion", default=None
    )

    span_id: Optional[str] = FieldInfo(alias="spanId", default=None)

    trace_id: Optional[str] = FieldInfo(alias="traceId", default=None)

    truncated: Optional[bool] = None


class EventsEventWorkersUnionMember1DiagnosticsChannelEvent(BaseModel):
    channel: str

    message: str

    timestamp: float


class EventsEventWorkersUnionMember1ScriptVersion(BaseModel):
    id: Optional[str] = None

    message: Optional[str] = None

    tag: Optional[str] = None


class EventsEventWorkersUnionMember1(BaseModel):
    cpu_time_ms: float = FieldInfo(alias="cpuTimeMs")

    event_type: Literal[
        "fetch", "scheduled", "alarm", "cron", "queue", "email", "tail", "rpc", "websocket", "workflow", "unknown"
    ] = FieldInfo(alias="eventType")

    outcome: str

    request_id: str = FieldInfo(alias="requestId")

    script_name: str = FieldInfo(alias="scriptName")

    wall_time_ms: float = FieldInfo(alias="wallTimeMs")

    diagnostics_channel_events: Optional[List[EventsEventWorkersUnionMember1DiagnosticsChannelEvent]] = FieldInfo(
        alias="diagnosticsChannelEvents", default=None
    )

    dispatch_namespace: Optional[str] = FieldInfo(alias="dispatchNamespace", default=None)

    durable_object_id: Optional[str] = FieldInfo(alias="durableObjectId", default=None)

    entrypoint: Optional[str] = None

    event: Optional[Dict[str, object]] = None

    execution_model: Optional[Literal["durableObject", "stateless"]] = FieldInfo(alias="executionModel", default=None)

    script_version: Optional[EventsEventWorkersUnionMember1ScriptVersion] = FieldInfo(
        alias="scriptVersion", default=None
    )

    span_id: Optional[str] = FieldInfo(alias="spanId", default=None)

    trace_id: Optional[str] = FieldInfo(alias="traceId", default=None)

    truncated: Optional[bool] = None


EventsEventWorkers: TypeAlias = Union[EventsEventWorkersUnionMember0, EventsEventWorkersUnionMember1]


class EventsEvent(BaseModel):
    """
    A single telemetry event representing a log line, span, or metric data point emitted by a Worker.
    """

    metadata: EventsEventMetadata = FieldInfo(alias="$metadata")
    """Structured metadata extracted from the event.

    These fields are indexed and available for filtering and aggregation.
    """

    dataset: str
    """The dataset this event belongs to (e.g. cloudflare-workers)."""

    source: Union[str, object]
    """Raw log payload.

    May be a string or a structured object depending on how the log was emitted.
    """

    timestamp: int
    """Event timestamp as a Unix epoch in milliseconds."""

    containers: Optional[object] = FieldInfo(alias="$containers", default=None)
    """
    Cloudflare Containers event information that enriches your logs for identifying
    and debugging issues.
    """

    workers: Optional[EventsEventWorkers] = FieldInfo(alias="$workers", default=None)
    """
    Cloudflare Workers event information that enriches your logs for identifying and
    debugging issues.
    """


class EventsField(BaseModel):
    key: str
    """Field name present in the matched events."""

    type: str
    """Data type of the field (string, number, or boolean)."""


class EventsSeriesDataAggregates(BaseModel):
    api_count: int = FieldInfo(alias="_count")

    api_interval: float = FieldInfo(alias="_interval")

    api_first_seen: Optional[str] = FieldInfo(alias="_firstSeen", default=None)

    api_last_seen: Optional[str] = FieldInfo(alias="_lastSeen", default=None)

    bin: Optional[object] = None


class EventsSeriesData(BaseModel):
    aggregates: EventsSeriesDataAggregates

    count: float

    interval: float

    sample_interval: float = FieldInfo(alias="sampleInterval")

    errors: Optional[float] = None

    groups: Optional[Dict[str, Union[str, float, bool]]] = None
    """Groups in the query results."""


class EventsSeries(BaseModel):
    data: List[EventsSeriesData]

    time: str


class Events(BaseModel):
    """Individual event results.

    Present when the query view is 'events'. Contains the matching log lines and their metadata.
    """

    count: Optional[float] = None
    """
    Total number of events matching the query (may exceed the number returned due to
    limits).
    """

    events: Optional[List[EventsEvent]] = None
    """List of individual telemetry events matching the query."""

    fields: Optional[List[EventsField]] = None
    """List of fields discovered in the matched events.

    Useful for building dynamic UIs.
    """

    series: Optional[List[EventsSeries]] = None
    """Time-series data for the matched events, bucketed by the query granularity."""


class InvocationMetadata(BaseModel):
    """Structured metadata extracted from the event.

    These fields are indexed and available for filtering and aggregation.
    """

    id: str
    """Unique event ID. Use as the cursor value for offset-based pagination."""

    account: Optional[str] = None
    """Cloudflare account identifier."""

    cloud_service: Optional[str] = FieldInfo(alias="cloudService", default=None)
    """Cloudflare product that generated this event (e.g. workers, pages)."""

    cold_start: Optional[int] = FieldInfo(alias="coldStart", default=None)
    """Whether this was a cold start (1) or warm invocation (0)."""

    cost: Optional[int] = None
    """Estimated cost units for this invocation."""

    duration: Optional[int] = None
    """Span duration in milliseconds."""

    end_time: Optional[int] = FieldInfo(alias="endTime", default=None)
    """Span end time as a Unix epoch in milliseconds."""

    error: Optional[str] = None
    """Error message, present when the log represents an error."""

    error_template: Optional[str] = FieldInfo(alias="errorTemplate", default=None)
    """Templatized version of the error message used for grouping similar errors."""

    fingerprint: Optional[str] = None
    """Content-based fingerprint used to group similar events."""

    level: Optional[str] = None
    """Log level (e.g. log, debug, info, warn, error)."""

    message: Optional[str] = None
    """Log message text."""

    message_template: Optional[str] = FieldInfo(alias="messageTemplate", default=None)
    """Templatized version of the log message used for grouping similar messages."""

    metric_name: Optional[str] = FieldInfo(alias="metricName", default=None)
    """Metric name when the event represents a metric data point."""

    origin: Optional[str] = None
    """Origin of the event (e.g. fetch, scheduled, queue)."""

    parent_span_id: Optional[str] = FieldInfo(alias="parentSpanId", default=None)
    """Span ID of the parent span in the trace hierarchy."""

    provider: Optional[str] = None
    """Infrastructure provider identifier."""

    region: Optional[str] = None
    """Cloudflare data center / region that handled the request."""

    request_id: Optional[str] = FieldInfo(alias="requestId", default=None)
    """Cloudflare request ID that ties all logs from a single invocation together."""

    service: Optional[str] = None
    """Worker script name that produced this event."""

    span_id: Optional[str] = FieldInfo(alias="spanId", default=None)
    """Span ID for this individual unit of work within a trace."""

    span_name: Optional[str] = FieldInfo(alias="spanName", default=None)
    """Human-readable name for this span."""

    stack_id: Optional[str] = FieldInfo(alias="stackId", default=None)
    """Stack / deployment identifier."""

    start_time: Optional[int] = FieldInfo(alias="startTime", default=None)
    """Span start time as a Unix epoch in milliseconds."""

    status_code: Optional[int] = FieldInfo(alias="statusCode", default=None)
    """HTTP response status code returned by the Worker."""

    trace_duration: Optional[int] = FieldInfo(alias="traceDuration", default=None)
    """Total duration of the entire trace in milliseconds."""

    trace_id: Optional[str] = FieldInfo(alias="traceId", default=None)
    """Distributed trace ID linking spans across services."""

    transaction_name: Optional[str] = FieldInfo(alias="transactionName", default=None)
    """Logical transaction name for this request."""

    trigger: Optional[str] = None
    """What triggered the invocation (e.g. GET /users, POST /orders, queue message)."""

    type: Optional[str] = None
    """Event type classifier (e.g. cf-worker-event, cf-worker-log)."""

    url: Optional[str] = None
    """Request URL that triggered the Worker invocation."""


class InvocationWorkersUnionMember0ScriptVersion(BaseModel):
    id: Optional[str] = None

    message: Optional[str] = None

    tag: Optional[str] = None


class InvocationWorkersUnionMember0(BaseModel):
    event_type: Literal[
        "fetch", "scheduled", "alarm", "cron", "queue", "email", "tail", "rpc", "websocket", "workflow", "unknown"
    ] = FieldInfo(alias="eventType")

    request_id: str = FieldInfo(alias="requestId")

    script_name: str = FieldInfo(alias="scriptName")

    durable_object_id: Optional[str] = FieldInfo(alias="durableObjectId", default=None)

    entrypoint: Optional[str] = None

    event: Optional[Dict[str, object]] = None

    execution_model: Optional[Literal["durableObject", "stateless"]] = FieldInfo(alias="executionModel", default=None)

    outcome: Optional[str] = None

    script_version: Optional[InvocationWorkersUnionMember0ScriptVersion] = FieldInfo(
        alias="scriptVersion", default=None
    )

    span_id: Optional[str] = FieldInfo(alias="spanId", default=None)

    trace_id: Optional[str] = FieldInfo(alias="traceId", default=None)

    truncated: Optional[bool] = None


class InvocationWorkersUnionMember1DiagnosticsChannelEvent(BaseModel):
    channel: str

    message: str

    timestamp: float


class InvocationWorkersUnionMember1ScriptVersion(BaseModel):
    id: Optional[str] = None

    message: Optional[str] = None

    tag: Optional[str] = None


class InvocationWorkersUnionMember1(BaseModel):
    cpu_time_ms: float = FieldInfo(alias="cpuTimeMs")

    event_type: Literal[
        "fetch", "scheduled", "alarm", "cron", "queue", "email", "tail", "rpc", "websocket", "workflow", "unknown"
    ] = FieldInfo(alias="eventType")

    outcome: str

    request_id: str = FieldInfo(alias="requestId")

    script_name: str = FieldInfo(alias="scriptName")

    wall_time_ms: float = FieldInfo(alias="wallTimeMs")

    diagnostics_channel_events: Optional[List[InvocationWorkersUnionMember1DiagnosticsChannelEvent]] = FieldInfo(
        alias="diagnosticsChannelEvents", default=None
    )

    dispatch_namespace: Optional[str] = FieldInfo(alias="dispatchNamespace", default=None)

    durable_object_id: Optional[str] = FieldInfo(alias="durableObjectId", default=None)

    entrypoint: Optional[str] = None

    event: Optional[Dict[str, object]] = None

    execution_model: Optional[Literal["durableObject", "stateless"]] = FieldInfo(alias="executionModel", default=None)

    script_version: Optional[InvocationWorkersUnionMember1ScriptVersion] = FieldInfo(
        alias="scriptVersion", default=None
    )

    span_id: Optional[str] = FieldInfo(alias="spanId", default=None)

    trace_id: Optional[str] = FieldInfo(alias="traceId", default=None)

    truncated: Optional[bool] = None


InvocationWorkers: TypeAlias = Union[InvocationWorkersUnionMember0, InvocationWorkersUnionMember1]


class Invocation(BaseModel):
    """
    A single telemetry event representing a log line, span, or metric data point emitted by a Worker.
    """

    metadata: InvocationMetadata = FieldInfo(alias="$metadata")
    """Structured metadata extracted from the event.

    These fields are indexed and available for filtering and aggregation.
    """

    dataset: str
    """The dataset this event belongs to (e.g. cloudflare-workers)."""

    source: Union[str, object]
    """Raw log payload.

    May be a string or a structured object depending on how the log was emitted.
    """

    timestamp: int
    """Event timestamp as a Unix epoch in milliseconds."""

    containers: Optional[object] = FieldInfo(alias="$containers", default=None)
    """
    Cloudflare Containers event information that enriches your logs for identifying
    and debugging issues.
    """

    workers: Optional[InvocationWorkers] = FieldInfo(alias="$workers", default=None)
    """
    Cloudflare Workers event information that enriches your logs for identifying and
    debugging issues.
    """


class Trace(BaseModel):
    root_span_name: str = FieldInfo(alias="rootSpanName")
    """Name of the root span that initiated the trace."""

    root_transaction_name: str = FieldInfo(alias="rootTransactionName")
    """Logical transaction name for the root span."""

    service: List[str]
    """List of Worker services involved in the trace."""

    spans: float
    """Total number of spans in the trace."""

    trace_duration_ms: float = FieldInfo(alias="traceDurationMs")
    """Total duration of the trace in milliseconds."""

    trace_end_ms: float = FieldInfo(alias="traceEndMs")
    """Trace end time as a Unix epoch in milliseconds."""

    trace_id: str = FieldInfo(alias="traceId")
    """Unique identifier for the distributed trace."""

    trace_start_ms: float = FieldInfo(alias="traceStartMs")
    """Trace start time as a Unix epoch in milliseconds."""

    errors: Optional[List[str]] = None
    """Error messages encountered during the trace, if any."""


class TelemetryQueryResponse(BaseModel):
    """Complete results of a query run.

    The populated fields depend on the requested view type (events, calculations, invocations, traces, or agents).
    """

    run: Run
    """
    The query run metadata including the query definition, execution status, and
    timeframe.
    """

    statistics: Statistics
    """Query performance statistics from the database.

    Includes execution time, rows scanned, and bytes read. Does not include network
    latency.
    """

    agents: Optional[List[Agent]] = None
    """Durable Object agent summaries.

    Present when the query view is 'agents'. Each entry represents an agent with its
    event counts and status.
    """

    calculations: Optional[List[Calculation]] = None
    """Aggregated calculation results.

    Present when the query view is 'calculations'. Contains computed metrics (count,
    avg, p99, etc.) with optional group-by breakdowns and time-series data.
    """

    compare: Optional[List[Compare]] = None
    """Comparison calculation results from the previous time period.

    Present when the compare option is enabled. Same structure as calculations.
    """

    events: Optional[Events] = None
    """Individual event results.

    Present when the query view is 'events'. Contains the matching log lines and
    their metadata.
    """

    invocations: Optional[Dict[str, List[Invocation]]] = None
    """Events grouped by invocation (request ID).

    Present when the query view is 'invocations'. Each key is a request ID mapping
    to all events from that invocation.
    """

    traces: Optional[List[Trace]] = None
    """Trace summaries matching the query.

    Present when the query view is 'traces'. Each entry represents a distributed
    trace with its spans, duration, and services involved.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ......_types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ......_utils import path_template, maybe_transform, async_maybe_transform
from ......_compat import cached_property
from ......_resource import SyncAPIResource, AsyncAPIResource
from ......_response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ......_wrappers import ResultWrapper
from ......_base_client import make_request_options
from ......types.ddos_protection.advanced_tcp_protection.syn_protection.filters import item_edit_params
from ......types.ddos_protection.advanced_tcp_protection.syn_protection.filters.item_get_response import ItemGetResponse
from ......types.ddos_protection.advanced_tcp_protection.syn_protection.filters.item_edit_response import (
    ItemEditResponse,
)
from ......types.ddos_protection.advanced_tcp_protection.syn_protection.filters.item_delete_response import (
    ItemDeleteResponse,
)

__all__ = ["ItemsResource", "AsyncItemsResource"]


class ItemsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ItemsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return ItemsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ItemsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return ItemsResourceWithStreamingResponse(self)

    def delete(
        self,
        filter_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ItemDeleteResponse:
        """
        Delete a SYN Protection filter specified by the given UUID.

        Args:
          account_id: Identifier.

          filter_id: UUID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not filter_id:
            raise ValueError(f"Expected a non-empty value for `filter_id` but received {filter_id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/magic/advanced_tcp_protection/configs/syn_protection/filters/{filter_id}",
                account_id=account_id,
                filter_id=filter_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ItemDeleteResponse,
        )

    def edit(
        self,
        filter_id: str,
        *,
        account_id: str,
        expression: str | Omit = omit,
        mode: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ItemEditResponse]:
        """
        Update a SYN Protection filter specified by the given UUID.

        Args:
          account_id: Identifier.

          filter_id: UUID.

          expression: The new filter expression. Optional.

          mode: The new mode for the filter. Optional. Must be one of 'enabled', 'disabled',
              'monitoring'.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not filter_id:
            raise ValueError(f"Expected a non-empty value for `filter_id` but received {filter_id!r}")
        return self._patch(
            path_template(
                "/accounts/{account_id}/magic/advanced_tcp_protection/configs/syn_protection/filters/{filter_id}",
                account_id=account_id,
                filter_id=filter_id,
            ),
            body=maybe_transform(
                {
                    "expression": expression,
                    "mode": mode,
                },
                item_edit_params.ItemEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[ItemEditResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[ItemEditResponse]], ResultWrapper[ItemEditResponse]),
        )

    def get(
        self,
        filter_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ItemGetResponse]:
        """
        Get a SYN Protection filter specified by the given UUID.

        Args:
          account_id: Identifier.

          filter_id: UUID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not filter_id:
            raise ValueError(f"Expected a non-empty value for `filter_id` but received {filter_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/magic/advanced_tcp_protection/configs/syn_protection/filters/{filter_id}",
                account_id=account_id,
                filter_id=filter_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[ItemGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[ItemGetResponse]], ResultWrapper[ItemGetResponse]),
        )


class AsyncItemsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncItemsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncItemsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncItemsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncItemsResourceWithStreamingResponse(self)

    async def delete(
        self,
        filter_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ItemDeleteResponse:
        """
        Delete a SYN Protection filter specified by the given UUID.

        Args:
          account_id: Identifier.

          filter_id: UUID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not filter_id:
            raise ValueError(f"Expected a non-empty value for `filter_id` but received {filter_id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/magic/advanced_tcp_protection/configs/syn_protection/filters/{filter_id}",
                account_id=account_id,
                filter_id=filter_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ItemDeleteResponse,
        )

    async def edit(
        self,
        filter_id: str,
        *,
        account_id: str,
        expression: str | Omit = omit,
        mode: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ItemEditResponse]:
        """
        Update a SYN Protection filter specified by the given UUID.

        Args:
          account_id: Identifier.

          filter_id: UUID.

          expression: The new filter expression. Optional.

          mode: The new mode for the filter. Optional. Must be one of 'enabled', 'disabled',
              'monitoring'.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not filter_id:
            raise ValueError(f"Expected a non-empty value for `filter_id` but received {filter_id!r}")
        return await self._patch(
            path_template(
                "/accounts/{account_id}/magic/advanced_tcp_protection/configs/syn_protection/filters/{filter_id}",
                account_id=account_id,
                filter_id=filter_id,
            ),
            body=await async_maybe_transform(
                {
                    "expression": expression,
                    "mode": mode,
                },
                item_edit_params.ItemEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[ItemEditResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[ItemEditResponse]], ResultWrapper[ItemEditResponse]),
        )

    async def get(
        self,
        filter_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ItemGetResponse]:
        """
        Get a SYN Protection filter specified by the given UUID.

        Args:
          account_id: Identifier.

          filter_id: UUID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not filter_id:
            raise ValueError(f"Expected a non-empty value for `filter_id` but received {filter_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/magic/advanced_tcp_protection/configs/syn_protection/filters/{filter_id}",
                account_id=account_id,
                filter_id=filter_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[ItemGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[ItemGetResponse]], ResultWrapper[ItemGetResponse]),
        )


class ItemsResourceWithRawResponse:
    def __init__(self, items: ItemsResource) -> None:
        self._items = items

        self.delete = to_raw_response_wrapper(
            items.delete,
        )
        self.edit = to_raw_response_wrapper(
            items.edit,
        )
        self.get = to_raw_response_wrapper(
            items.get,
        )


class AsyncItemsResourceWithRawResponse:
    def __init__(self, items: AsyncItemsResource) -> None:
        self._items = items

        self.delete = async_to_raw_response_wrapper(
            items.delete,
        )
        self.edit = async_to_raw_response_wrapper(
            items.edit,
        )
        self.get = async_to_raw_response_wrapper(
            items.get,
        )


class ItemsResourceWithStreamingResponse:
    def __init__(self, items: ItemsResource) -> None:
        self._items = items

        self.delete = to_streamed_response_wrapper(
            items.delete,
        )
        self.edit = to_streamed_response_wrapper(
            items.edit,
        )
        self.get = to_streamed_response_wrapper(
            items.get,
        )


class AsyncItemsResourceWithStreamingResponse:
    def __init__(self, items: AsyncItemsResource) -> None:
        self._items = items

        self.delete = async_to_streamed_response_wrapper(
            items.delete,
        )
        self.edit = async_to_streamed_response_wrapper(
            items.edit,
        )
        self.get = async_to_streamed_response_wrapper(
            items.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel
from .additional_routes import AdditionalRoutes
from .cookie_attributes import CookieAttributes

__all__ = ["WaitingRoom"]


class WaitingRoom(BaseModel):
    id: Optional[str] = None

    additional_routes: Optional[List[AdditionalRoutes]] = None
    """Only available for the Waiting Room Advanced subscription.

    Additional hostname and path combinations to which this waiting room will be
    applied. There is an implied wildcard at the end of the path. The hostname and
    path combination must be unique to this and all other waiting rooms.
    """

    cookie_attributes: Optional[CookieAttributes] = None
    """Configures cookie attributes for the waiting room cookie.

    This encrypted cookie stores a user's status in the waiting room, such as queue
    position.
    """

    cookie_suffix: Optional[str] = None
    """
    Appends a '\\__' + a custom suffix to the end of Cloudflare Waiting Room's cookie
    name(**cf_waitingroom). If `cookie_suffix` is "abcd", the cookie name will be
    `**cf_waitingroom_abcd`. This field is required if using `additional_routes`.
    """

    created_on: Optional[datetime] = None

    custom_page_html: Optional[str] = None
    """Only available for the Waiting Room Advanced subscription.

    This is a template html file that will be rendered at the edge. If no
    custom_page_html is provided, the default waiting room will be used. The
    template is based on mustache ( https://mustache.github.io/ ). There are several
    variables that are evaluated by the Cloudflare edge:

    1. {{`waitTimeKnown`}} Acts like a boolean value that indicates the behavior to
       take when wait time is not available, for instance when queue_all is
       **true**.
    2. {{`waitTimeFormatted`}} Estimated wait time for the user. For example, five
       minutes. Alternatively, you can use:
    3. {{`waitTime`}} Number of minutes of estimated wait for a user.
    4. {{`waitTimeHours`}} Number of hours of estimated wait for a user
       (`Math.floor(waitTime/60)`).
    5. {{`waitTimeHourMinutes`}} Number of minutes above the `waitTimeHours` value
       (`waitTime%60`).
    6. {{`queueIsFull`}} Changes to **true** when no more people can be added to the
       queue.

    To view the full list of variables, look at the `cfWaitingRoom` object described
    under the `json_response_enabled` property in other Waiting Room API calls.
    """

    default_template_language: Optional[
        Literal[
            "en-US",
            "es-ES",
            "de-DE",
            "fr-FR",
            "it-IT",
            "ja-JP",
            "ko-KR",
            "pt-BR",
            "zh-CN",
            "zh-TW",
            "nl-NL",
            "pl-PL",
            "id-ID",
            "tr-TR",
            "ar-EG",
            "ru-RU",
            "fa-IR",
            "bg-BG",
            "hr-HR",
            "cs-CZ",
            "da-DK",
            "fi-FI",
            "lt-LT",
            "ms-MY",
            "nb-NO",
            "ro-RO",
            "el-GR",
            "he-IL",
            "hi-IN",
            "hu-HU",
            "sr-BA",
            "sk-SK",
            "sl-SI",
            "sv-SE",
            "tl-PH",
            "th-TH",
            "uk-UA",
            "vi-VN",
        ]
    ] = None
    """The language of the default page template.

    If no default_template_language is provided, then `en-US` (English) will be
    used.
    """

    description: Optional[str] = None
    """A note that you can use to add more details about the waiting room."""

    disable_session_renewal: Optional[bool] = None
    """Only available for the Waiting Room Advanced subscription.

    Disables automatic renewal of session cookies. If `true`, an accepted user will
    have session_duration minutes to browse the site. After that, they will have to
    go through the waiting room again. If `false`, a user's session cookie will be
    automatically renewed on every request.
    """

    enabled_origin_commands: Optional[List[Literal["revoke"]]] = None
    """A list of enabled origin commands."""

    host: Optional[str] = None
    """The host name to which the waiting room will be applied (no wildcards).

    Please do not include the scheme (http:// or https://). The host and path
    combination must be unique.
    """

    json_response_enabled: Optional[bool] = None
    """Only available for the Waiting Room Advanced subscription.

    If `true`, requests to the waiting room with the header
    `Accept: application/json` will receive a JSON response object with information
    on the user's status in the waiting room as opposed to the configured static
    HTML page. This JSON response object has one property `cfWaitingRoom` which is
    an object containing the following fields:

    1. `inWaitingRoom`: Boolean indicating if the user is in the waiting room
       (always **true**).
    2. `waitTimeKnown`: Boolean indicating if the current estimated wait times are
       accurate. If **false**, they are not available.
    3. `waitTime`: Valid only when `waitTimeKnown` is **true**. Integer indicating
       the current estimated time in minutes the user will wait in the waiting room.
       When `queueingMethod` is **random**, this is set to `waitTime50Percentile`.
    4. `waitTime25Percentile`: Valid only when `queueingMethod` is **random** and
       `waitTimeKnown` is **true**. Integer indicating the current estimated maximum
       wait time for the 25% of users that gain entry the fastest (25th percentile).
    5. `waitTime50Percentile`: Valid only when `queueingMethod` is **random** and
       `waitTimeKnown` is **true**. Integer indicating the current estimated maximum
       wait time for the 50% of users that gain entry the fastest (50th percentile).
       In other words, half of the queued users are expected to let into the origin
       website before `waitTime50Percentile` and half are expected to be let in
       after it.
    6. `waitTime75Percentile`: Valid only when `queueingMethod` is **random** and
       `waitTimeKnown` is **true**. Integer indicating the current estimated maximum
       wait time for the 75% of users that gain entry the fastest (75th percentile).
    7. `waitTimeFormatted`: String displaying the `waitTime` formatted in English
       for users. If `waitTimeKnown` is **false**, `waitTimeFormatted` will display
       **unavailable**.
    8. `queueIsFull`: Boolean indicating if the waiting room's queue is currently
       full and not accepting new users at the moment.
    9. `queueAll`: Boolean indicating if all users will be queued in the waiting
       room and no one will be let into the origin website.
    10. `lastUpdated`: String displaying the timestamp as an ISO 8601 string of the
        user's last attempt to leave the waiting room and be let into the origin
        website. The user is able to make another attempt after
        `refreshIntervalSeconds` past this time. If the user makes a request too
        soon, it will be ignored and `lastUpdated` will not change.
    11. `refreshIntervalSeconds`: Integer indicating the number of seconds after
        `lastUpdated` until the user is able to make another attempt to leave the
        waiting room and be let into the origin website. When the `queueingMethod`
        is `reject`, there is no specified refresh time —\\__it will always be
        **zero**.
    12. `queueingMethod`: The queueing method currently used by the waiting room. It
        is either **fifo**, **random**, **passthrough**, or **reject**.
    13. `isFIFOQueue`: Boolean indicating if the waiting room uses a FIFO
        (First-In-First-Out) queue.
    14. `isRandomQueue`: Boolean indicating if the waiting room uses a Random queue
        where users gain access randomly.
    15. `isPassthroughQueue`: Boolean indicating if the waiting room uses a
        passthrough queue. Keep in mind that when passthrough is enabled, this JSON
        response will only exist when `queueAll` is **true** or `isEventPrequeueing`
        is **true** because in all other cases requests will go directly to the
        origin.
    16. `isRejectQueue`: Boolean indicating if the waiting room uses a reject queue.
    17. `isEventActive`: Boolean indicating if an event is currently occurring.
        Events are able to change a waiting room's behavior during a specified
        period of time. For additional information, look at the event properties
        `prequeue_start_time`, `event_start_time`, and `event_end_time` in the
        documentation for creating waiting room events. Events are considered active
        between these start and end times, as well as during the prequeueing period
        if it exists.
    18. `isEventPrequeueing`: Valid only when `isEventActive` is **true**. Boolean
        indicating if an event is currently prequeueing users before it starts.
    19. `timeUntilEventStart`: Valid only when `isEventPrequeueing` is **true**.
        Integer indicating the number of minutes until the event starts.
    20. `timeUntilEventStartFormatted`: String displaying the `timeUntilEventStart`
        formatted in English for users. If `isEventPrequeueing` is **false**,
        `timeUntilEventStartFormatted` will display **unavailable**.
    21. `timeUntilEventEnd`: Valid only when `isEventActive` is **true**. Integer
        indicating the number of minutes until the event ends.
    22. `timeUntilEventEndFormatted`: String displaying the `timeUntilEventEnd`
        formatted in English for users. If `isEventActive` is **false**,
        `timeUntilEventEndFormatted` will display **unavailable**.
    23. `shuffleAtEventStart`: Valid only when `isEventActive` is **true**. Boolean
        indicating if the users in the prequeue are shuffled randomly when the event
        starts.
    24. `turnstile`: Empty when turnstile isn't enabled. String displaying an html
        tag to display the Turnstile widget. Please add the `{{{turnstile}}}` tag to
        the `custom_html` template to ensure the Turnstile widget appears.
    25. `infiniteQueue`: Boolean indicating whether the response is for a user in
        the infinite queue.

    An example cURL to a waiting room could be:

        curl -X GET "https://example.com/waitingroom" \\
        	-H "Accept: application/json"

    If `json_response_enabled` is **true** and the request hits the waiting room, an
    example JSON response when `queueingMethod` is **fifo** and no event is active
    could be:

        {
        	"cfWaitingRoom": {
        		"inWaitingRoom": true,
        		"waitTimeKnown": true,
        		"waitTime": 10,
        		"waitTime25Percentile": 0,
        		"waitTime50Percentile": 0,
        		"waitTime75Percentile": 0,
        		"waitTimeFormatted": "10 minutes",
        		"queueIsFull": false,
        		"queueAll": false,
        		"lastUpdated": "2020-08-03T23:46:00.000Z",
        		"refreshIntervalSeconds": 20,
        		"queueingMethod": "fifo",
        		"isFIFOQueue": true,
        		"isRandomQueue": false,
        		"isPassthroughQueue": false,
        		"isRejectQueue": false,
        		"isEventActive": false,
        		"isEventPrequeueing": false,
        		"timeUntilEventStart": 0,
        		"timeUntilEventStartFormatted": "unavailable",
        		"timeUntilEventEnd": 0,
        		"timeUntilEventEndFormatted": "unavailable",
        		"shuffleAtEventStart": false
        	}
        }

    If `json_response_enabled` is **true** and the request hits the waiting room, an
    example JSON response when `queueingMethod` is **random** and an event is active
    could be:

        {
        	"cfWaitingRoom": {
        		"inWaitingRoom": true,
        		"waitTimeKnown": true,
        		"waitTime": 10,
        		"waitTime25Percentile": 5,
        		"waitTime50Percentile": 10,
        		"waitTime75Percentile": 15,
        		"waitTimeFormatted": "5 minutes to 15 minutes",
        		"queueIsFull": false,
        		"queueAll": false,
        		"lastUpdated": "2020-08-03T23:46:00.000Z",
        		"refreshIntervalSeconds": 20,
        		"queueingMethod": "random",
        		"isFIFOQueue": false,
        		"isRandomQueue": true,
        		"isPassthroughQueue": false,
        		"isRejectQueue": false,
        		"isEventActive": true,
        		"isEventPrequeueing": false,
        		"timeUntilEventStart": 0,
        		"timeUntilEventStartFormatted": "unavailable",
        		"timeUntilEventEnd": 15,
        		"timeUntilEventEndFormatted": "15 minutes",
        		"shuffleAtEventStart": true
        	}
        }
    """

    modified_on: Optional[datetime] = None

    name: Optional[str] = None
    """A unique name to identify the waiting room.

    Only alphanumeric characters, hyphens and underscores are allowed.
    """

    new_users_per_minute: Optional[int] = None
    """Sets the number of new users that will be let into the route every minute.

    This value is used as baseline for the number of users that are let in per
    minute. So it is possible that there is a little more or little less traffic
    coming to the route based on the traffic patterns at that time around the world.
    """

    next_event_prequeue_start_time: Optional[str] = None
    """An ISO 8601 timestamp that marks when the next event will begin queueing."""

    next_event_start_time: Optional[str] = None
    """An ISO 8601 timestamp that marks when the next event will start."""

    path: Optional[str] = None
    """Sets the path within the host to enable the waiting room on.

    The waiting room will be enabled for all subpaths as well. If there are two
    waiting rooms on the same subpath, the waiting room for the most specific path
    will be chosen. Wildcards and query parameters are not supported.
    """

    queue_all: Optional[bool] = None
    """
    If queue_all is `true`, all the traffic that is coming to a route will be sent
    to the waiting room. No new traffic can get to the route once this field is set
    and estimated time will become unavailable.
    """

    queueing_method: Optional[Literal["fifo", "random", "passthrough", "reject"]] = None
    """Sets the queueing method used by the waiting room.

    Changing this parameter from the **default** queueing method is only available
    for the Waiting Room Advanced subscription. Regardless of the queueing method,
    if `queue_all` is enabled or an event is prequeueing, users in the waiting room
    will not be accepted to the origin. These users will always see a waiting room
    page that refreshes automatically. The valid queueing methods are:

    1. `fifo` **(default)**: First-In-First-Out queue where customers gain access in
       the order they arrived.
    2. `random`: Random queue where customers gain access randomly, regardless of
       arrival time.
    3. `passthrough`: Users will pass directly through the waiting room and into the
       origin website. As a result, any configured limits will not be respected
       while this is enabled. This method can be used as an alternative to disabling
       a waiting room (with `suspended`) so that analytics are still reported. This
       can be used if you wish to allow all traffic normally, but want to restrict
       traffic during a waiting room event, or vice versa.
    4. `reject`: Users will be immediately rejected from the waiting room. As a
       result, no users will reach the origin website while this is enabled. This
       can be used if you wish to reject all traffic while performing maintenance,
       block traffic during a specified period of time (an event), or block traffic
       while events are not occurring. Consider a waiting room used for vaccine
       distribution that only allows traffic during sign-up events, and otherwise
       blocks all traffic. For this case, the waiting room uses `reject`, and its
       events override this with `fifo`, `random`, or `passthrough`. When this
       queueing method is enabled and neither `queueAll` is enabled nor an event is
       prequeueing, the waiting room page **will not refresh automatically**.
    """

    queueing_status_code: Optional[Literal[200, 202, 429]] = None
    """HTTP status code returned to a user while in the queue."""

    session_duration: Optional[int] = None
    """
    Lifetime of a cookie (in minutes) set by Cloudflare for users who get access to
    the route. If a user is not seen by Cloudflare again in that time period, they
    will be treated as a new user that visits the route.
    """

    suspended: Optional[bool] = None
    """Suspends or allows traffic going to the waiting room.

    If set to `true`, the traffic will not go to the waiting room.
    """

    total_active_users: Optional[int] = None
    """Sets the total number of active user sessions on the route at a point in time.

    A route is a combination of host and path on which a waiting room is available.
    This value is used as a baseline for the total number of active user sessions on
    the route. It is possible to have a situation where there are more or less
    active users sessions on the route based on the traffic patterns at that time
    around the world.
    """

    turnstile_action: Optional[Literal["log", "infinite_queue"]] = None
    """Which action to take when a bot is detected using Turnstile.

    `log` will have no impact on queueing behavior, simply keeping track of how many
    bots are detected in Waiting Room Analytics. `infinite_queue` will send bots to
    a false queueing state, where they will never reach your origin.
    `infinite_queue` requires Advanced Waiting Room.
    """

    turnstile_mode: Optional[Literal["off", "invisible", "visible_non_interactive", "visible_managed"]] = None
    """Which Turnstile widget type to use for detecting bot traffic.

    See
    [the Turnstile documentation](https://developers.cloudflare.com/turnstile/concepts/widget/#widget-types)
    for the definitions of these widget types. Set to `off` to disable the Turnstile
    integration entirely. Setting this to anything other than `off` or `invisible`
    requires Advanced Waiting Room.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, TypeAlias

from .dns import DNS
from .edge_ips import EdgeIPs
from ..._models import BaseModel
from .origin_dns import OriginDNS
from .origin_port import OriginPort

__all__ = ["AppListResponse", "SpectrumConfigAppConfig", "SpectrumConfigPaygoAppConfig"]


class SpectrumConfigAppConfig(BaseModel):
    id: str
    """App identifier."""

    created_on: datetime
    """When the Application was created."""

    dns: DNS
    """The name and type of DNS record for the Spectrum application."""

    modified_on: datetime
    """When the Application was last modified."""

    protocol: str
    """The port configuration at Cloudflare's edge.

    May specify a single port, for example `"tcp/1000"`, or a range of ports, for
    example `"tcp/1000-2000"`.
    """

    traffic_type: Literal["direct", "http", "https"]
    """Determines how data travels from the edge to your origin.

    When set to "direct", Spectrum will send traffic directly to your origin, and
    the application's type is derived from the `protocol`. When set to "http" or
    "https", Spectrum will apply Cloudflare's HTTP/HTTPS features as it sends
    traffic to your origin, and the application type matches this property exactly.
    """

    argo_smart_routing: Optional[bool] = None
    """
    Enables Argo Smart Routing for this application. Notes: Only available for TCP
    applications with traffic_type set to "direct".
    """

    edge_ips: Optional[EdgeIPs] = None
    """The anycast edge IP configuration for the hostname of this application."""

    ip_firewall: Optional[bool] = None
    """
    Enables IP Access Rules for this application. Notes: Only available for TCP
    applications.
    """

    origin_direct: Optional[List[str]] = None
    """List of origin IP addresses.

    Array may contain multiple IP addresses for load balancing.
    """

    origin_dns: Optional[OriginDNS] = None
    """The name and type of DNS record for the Spectrum application."""

    origin_port: Optional[OriginPort] = None
    """The destination port at the origin.

    Only specified in conjunction with origin_dns. May use an integer to specify a
    single origin port, for example `1000`, or a string to specify a range of origin
    ports, for example `"1000-2000"`. Notes: If specifying a port range, the number
    of ports in the range must match the number of ports specified in the "protocol"
    field.
    """

    proxy_protocol: Optional[Literal["off", "v1", "v2", "simple"]] = None
    """Enables Proxy Protocol to the origin.

    Refer to
    [Enable Proxy protocol](https://developers.cloudflare.com/spectrum/getting-started/proxy-protocol/)
    for implementation details on PROXY Protocol V1, PROXY Protocol V2, and Simple
    Proxy Protocol.
    """

    tls: Optional[Literal["off", "flexible", "full", "strict"]] = None
    """The type of TLS termination associated with the application."""


class SpectrumConfigPaygoAppConfig(BaseModel):
    id: str
    """App identifier."""

    created_on: datetime
    """When the Application was created."""

    dns: DNS
    """The name and type of DNS record for the Spectrum application."""

    modified_on: datetime
    """When the Application was last modified."""

    protocol: str
    """The port configuration at Cloudflare's edge.

    May specify a single port, for example `"tcp/1000"`, or a range of ports, for
    example `"tcp/1000-2000"`.
    """

    origin_direct: Optional[List[str]] = None
    """List of origin IP addresses.

    Array may contain multiple IP addresses for load balancing.
    """


AppListResponse: TypeAlias = Union[SpectrumConfigAppConfig, SpectrumConfigPaygoAppConfig]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ...._models import BaseModel

__all__ = ["RuleGetResponse", "TargetedTest", "TargetedTestData"]


class TargetedTestData(BaseModel):
    """
    The configuration object which contains the details for the WARP client to conduct the test.
    """

    host: str
    """The desired endpoint to test."""

    kind: Literal["http", "traceroute"]
    """The type of test."""

    method: Optional[Literal["GET"]] = None
    """The HTTP request method type."""


class TargetedTest(BaseModel):
    data: TargetedTestData
    """
    The configuration object which contains the details for the WARP client to
    conduct the test.
    """

    enabled: bool

    name: str

    test_id: str


class RuleGetResponse(BaseModel):
    id: str
    """API Resource UUID tag."""

    created_at: str

    match: str

    name: str

    description: Optional[str] = None

    targeted_tests: Optional[List[TargetedTest]] = None

    updated_at: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from ..._models import BaseModel
from .labeled_region import LabeledRegion

__all__ = [
    "Availability",
    "Quota",
    "QuotaQuotasPerPlan",
    "QuotaQuotasPerPlanValue",
    "QuotaScheduleQuotasPerPlan",
    "QuotaScheduleQuotasPerPlanValue",
    "RegionsPerPlan",
]


class QuotaQuotasPerPlanValue(BaseModel):
    """Counts per account plan."""

    business: Optional[int] = None

    enterprise: Optional[int] = None

    free: Optional[int] = None

    pro: Optional[int] = None


class QuotaQuotasPerPlan(BaseModel):
    """The number of tests available per plan."""

    value: Optional[QuotaQuotasPerPlanValue] = None
    """Counts per account plan."""


class QuotaScheduleQuotasPerPlanValue(BaseModel):
    """Counts per account plan."""

    business: Optional[int] = None

    enterprise: Optional[int] = None

    free: Optional[int] = None

    pro: Optional[int] = None


class QuotaScheduleQuotasPerPlan(BaseModel):
    """The number of schedules available per plan."""

    value: Optional[QuotaScheduleQuotasPerPlanValue] = None
    """Counts per account plan."""


class Quota(BaseModel):
    plan: Optional[str] = None
    """Cloudflare plan."""

    quotas_per_plan: Optional[QuotaQuotasPerPlan] = FieldInfo(alias="quotasPerPlan", default=None)
    """The number of tests available per plan."""

    remaining_schedules: Optional[float] = FieldInfo(alias="remainingSchedules", default=None)
    """The number of remaining schedules available."""

    remaining_tests: Optional[float] = FieldInfo(alias="remainingTests", default=None)
    """The number of remaining tests available."""

    schedule_quotas_per_plan: Optional[QuotaScheduleQuotasPerPlan] = FieldInfo(
        alias="scheduleQuotasPerPlan", default=None
    )
    """The number of schedules available per plan."""


class RegionsPerPlan(BaseModel):
    """Available regions."""

    business: Optional[List[LabeledRegion]] = None

    enterprise: Optional[List[LabeledRegion]] = None

    free: Optional[List[LabeledRegion]] = None

    pro: Optional[List[LabeledRegion]] = None


class Availability(BaseModel):
    quota: Optional[Quota] = None

    regions: Optional[List[LabeledRegion]] = None

    regions_per_plan: Optional[RegionsPerPlan] = FieldInfo(alias="regionsPerPlan", default=None)
    """Available regions."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Union, Optional
from typing_extensions import Literal, Annotated, TypeAlias

from pydantic import Field as FieldInfo

from ..._utils import PropertyInfo
from ..._models import BaseModel

__all__ = [
    "ResourceGetResponse",
    "MonthlyCostEstimate",
    "Observations",
    "Section",
    "SectionHiddenItem",
    "SectionHiddenItemValue",
    "SectionHiddenItemValueMcnStringItem",
    "SectionHiddenItemValueMcnYamlItem",
    "SectionHiddenItemValueMcnYamlDiffItem",
    "SectionHiddenItemValueMcnYamlDiffItemYamlDiff",
    "SectionHiddenItemValueMcnResourcePreviewItem",
    "SectionHiddenItemValueMcnResourcePreviewItemResourcePreview",
    "SectionHiddenItemValueMcnListItem",
    "SectionHiddenItemValueMcnListItemList",
    "SectionHiddenItemValueMcnListItemListMcnStringItem",
    "SectionHiddenItemValueMcnListItemListMcnResourcePreviewItem",
    "SectionHiddenItemValueMcnListItemListMcnResourcePreviewItemResourcePreview",
    "SectionVisibleItem",
    "SectionVisibleItemValue",
    "SectionVisibleItemValueMcnStringItem",
    "SectionVisibleItemValueMcnYamlItem",
    "SectionVisibleItemValueMcnYamlDiffItem",
    "SectionVisibleItemValueMcnYamlDiffItemYamlDiff",
    "SectionVisibleItemValueMcnResourcePreviewItem",
    "SectionVisibleItemValueMcnResourcePreviewItemResourcePreview",
    "SectionVisibleItemValueMcnListItem",
    "SectionVisibleItemValueMcnListItemList",
    "SectionVisibleItemValueMcnListItemListMcnStringItem",
    "SectionVisibleItemValueMcnListItemListMcnResourcePreviewItem",
    "SectionVisibleItemValueMcnListItemListMcnResourcePreviewItemResourcePreview",
    "ManagedBy",
]


class MonthlyCostEstimate(BaseModel):
    currency: str

    monthly_cost: float


class Observations(BaseModel):
    first_observed_at: str

    last_observed_at: str

    provider_id: str

    resource_id: str


class SectionHiddenItemValueMcnStringItem(BaseModel):
    item_type: str

    string: str


class SectionHiddenItemValueMcnYamlItem(BaseModel):
    item_type: str

    yaml: str


class SectionHiddenItemValueMcnYamlDiffItemYamlDiff(BaseModel):
    diff: str

    left_description: str

    left_yaml: str

    right_description: str

    right_yaml: str


class SectionHiddenItemValueMcnYamlDiffItem(BaseModel):
    item_type: str

    yaml_diff: SectionHiddenItemValueMcnYamlDiffItemYamlDiff


class SectionHiddenItemValueMcnResourcePreviewItemResourcePreview(BaseModel):
    id: str

    cloud_type: Literal["AWS", "AZURE", "GOOGLE", "CLOUDFLARE"]

    detail: str

    name: str

    resource_type: Literal[
        "aws_customer_gateway",
        "aws_egress_only_internet_gateway",
        "aws_internet_gateway",
        "aws_instance",
        "aws_network_interface",
        "aws_route",
        "aws_route_table",
        "aws_route_table_association",
        "aws_subnet",
        "aws_vpc",
        "aws_vpc_ipv4_cidr_block_association",
        "aws_vpn_connection",
        "aws_vpn_connection_route",
        "aws_vpn_gateway",
        "aws_security_group",
        "aws_vpc_security_group_ingress_rule",
        "aws_vpc_security_group_egress_rule",
        "aws_ec2_managed_prefix_list",
        "aws_ec2_transit_gateway",
        "aws_ec2_transit_gateway_prefix_list_reference",
        "aws_ec2_transit_gateway_vpc_attachment",
        "azurerm_application_security_group",
        "azurerm_lb",
        "azurerm_lb_backend_address_pool",
        "azurerm_lb_nat_pool",
        "azurerm_lb_nat_rule",
        "azurerm_lb_rule",
        "azurerm_local_network_gateway",
        "azurerm_network_interface",
        "azurerm_network_interface_application_security_group_association",
        "azurerm_network_interface_backend_address_pool_association",
        "azurerm_network_interface_security_group_association",
        "azurerm_network_security_group",
        "azurerm_public_ip",
        "azurerm_route",
        "azurerm_route_table",
        "azurerm_subnet",
        "azurerm_subnet_route_table_association",
        "azurerm_virtual_machine",
        "azurerm_virtual_network_gateway_connection",
        "azurerm_virtual_network",
        "azurerm_virtual_network_gateway",
        "google_compute_network",
        "google_compute_subnetwork",
        "google_compute_vpn_gateway",
        "google_compute_vpn_tunnel",
        "google_compute_route",
        "google_compute_address",
        "google_compute_global_address",
        "google_compute_router",
        "google_compute_interconnect_attachment",
        "google_compute_ha_vpn_gateway",
        "google_compute_forwarding_rule",
        "google_compute_network_firewall_policy",
        "google_compute_network_firewall_policy_rule",
        "cloudflare_static_route",
        "cloudflare_ipsec_tunnel",
    ]

    title: str


class SectionHiddenItemValueMcnResourcePreviewItem(BaseModel):
    item_type: str

    resource_preview: SectionHiddenItemValueMcnResourcePreviewItemResourcePreview


class SectionHiddenItemValueMcnListItemListMcnStringItem(BaseModel):
    item_type: str

    string: str


class SectionHiddenItemValueMcnListItemListMcnResourcePreviewItemResourcePreview(BaseModel):
    id: str

    cloud_type: Literal["AWS", "AZURE", "GOOGLE", "CLOUDFLARE"]

    detail: str

    name: str

    resource_type: Literal[
        "aws_customer_gateway",
        "aws_egress_only_internet_gateway",
        "aws_internet_gateway",
        "aws_instance",
        "aws_network_interface",
        "aws_route",
        "aws_route_table",
        "aws_route_table_association",
        "aws_subnet",
        "aws_vpc",
        "aws_vpc_ipv4_cidr_block_association",
        "aws_vpn_connection",
        "aws_vpn_connection_route",
        "aws_vpn_gateway",
        "aws_security_group",
        "aws_vpc_security_group_ingress_rule",
        "aws_vpc_security_group_egress_rule",
        "aws_ec2_managed_prefix_list",
        "aws_ec2_transit_gateway",
        "aws_ec2_transit_gateway_prefix_list_reference",
        "aws_ec2_transit_gateway_vpc_attachment",
        "azurerm_application_security_group",
        "azurerm_lb",
        "azurerm_lb_backend_address_pool",
        "azurerm_lb_nat_pool",
        "azurerm_lb_nat_rule",
        "azurerm_lb_rule",
        "azurerm_local_network_gateway",
        "azurerm_network_interface",
        "azurerm_network_interface_application_security_group_association",
        "azurerm_network_interface_backend_address_pool_association",
        "azurerm_network_interface_security_group_association",
        "azurerm_network_security_group",
        "azurerm_public_ip",
        "azurerm_route",
        "azurerm_route_table",
        "azurerm_subnet",
        "azurerm_subnet_route_table_association",
        "azurerm_virtual_machine",
        "azurerm_virtual_network_gateway_connection",
        "azurerm_virtual_network",
        "azurerm_virtual_network_gateway",
        "google_compute_network",
        "google_compute_subnetwork",
        "google_compute_vpn_gateway",
        "google_compute_vpn_tunnel",
        "google_compute_route",
        "google_compute_address",
        "google_compute_global_address",
        "google_compute_router",
        "google_compute_interconnect_attachment",
        "google_compute_ha_vpn_gateway",
        "google_compute_forwarding_rule",
        "google_compute_network_firewall_policy",
        "google_compute_network_firewall_policy_rule",
        "cloudflare_static_route",
        "cloudflare_ipsec_tunnel",
    ]

    title: str


class SectionHiddenItemValueMcnListItemListMcnResourcePreviewItem(BaseModel):
    item_type: str

    resource_preview: SectionHiddenItemValueMcnListItemListMcnResourcePreviewItemResourcePreview


SectionHiddenItemValueMcnListItemList: TypeAlias = Annotated[
    Union[
        SectionHiddenItemValueMcnListItemListMcnStringItem, SectionHiddenItemValueMcnListItemListMcnResourcePreviewItem
    ],
    PropertyInfo(discriminator="item_type"),
]


class SectionHiddenItemValueMcnListItem(BaseModel):
    item_type: str

    rule_list: List[SectionHiddenItemValueMcnListItemList] = FieldInfo(alias="list")


SectionHiddenItemValue: TypeAlias = Annotated[
    Union[
        SectionHiddenItemValueMcnStringItem,
        SectionHiddenItemValueMcnYamlItem,
        SectionHiddenItemValueMcnYamlDiffItem,
        SectionHiddenItemValueMcnResourcePreviewItem,
        SectionHiddenItemValueMcnListItem,
    ],
    PropertyInfo(discriminator="item_type"),
]


class SectionHiddenItem(BaseModel):
    help_text: Optional[str] = FieldInfo(alias="helpText", default=None)

    name: Optional[str] = None

    value: Optional[SectionHiddenItemValue] = None


class SectionVisibleItemValueMcnStringItem(BaseModel):
    item_type: str

    string: str


class SectionVisibleItemValueMcnYamlItem(BaseModel):
    item_type: str

    yaml: str


class SectionVisibleItemValueMcnYamlDiffItemYamlDiff(BaseModel):
    diff: str

    left_description: str

    left_yaml: str

    right_description: str

    right_yaml: str


class SectionVisibleItemValueMcnYamlDiffItem(BaseModel):
    item_type: str

    yaml_diff: SectionVisibleItemValueMcnYamlDiffItemYamlDiff


class SectionVisibleItemValueMcnResourcePreviewItemResourcePreview(BaseModel):
    id: str

    cloud_type: Literal["AWS", "AZURE", "GOOGLE", "CLOUDFLARE"]

    detail: str

    name: str

    resource_type: Literal[
        "aws_customer_gateway",
        "aws_egress_only_internet_gateway",
        "aws_internet_gateway",
        "aws_instance",
        "aws_network_interface",
        "aws_route",
        "aws_route_table",
        "aws_route_table_association",
        "aws_subnet",
        "aws_vpc",
        "aws_vpc_ipv4_cidr_block_association",
        "aws_vpn_connection",
        "aws_vpn_connection_route",
        "aws_vpn_gateway",
        "aws_security_group",
        "aws_vpc_security_group_ingress_rule",
        "aws_vpc_security_group_egress_rule",
        "aws_ec2_managed_prefix_list",
        "aws_ec2_transit_gateway",
        "aws_ec2_transit_gateway_prefix_list_reference",
        "aws_ec2_transit_gateway_vpc_attachment",
        "azurerm_application_security_group",
        "azurerm_lb",
        "azurerm_lb_backend_address_pool",
        "azurerm_lb_nat_pool",
        "azurerm_lb_nat_rule",
        "azurerm_lb_rule",
        "azurerm_local_network_gateway",
        "azurerm_network_interface",
        "azurerm_network_interface_application_security_group_association",
        "azurerm_network_interface_backend_address_pool_association",
        "azurerm_network_interface_security_group_association",
        "azurerm_network_security_group",
        "azurerm_public_ip",
        "azurerm_route",
        "azurerm_route_table",
        "azurerm_subnet",
        "azurerm_subnet_route_table_association",
        "azurerm_virtual_machine",
        "azurerm_virtual_network_gateway_connection",
        "azurerm_virtual_network",
        "azurerm_virtual_network_gateway",
        "google_compute_network",
        "google_compute_subnetwork",
        "google_compute_vpn_gateway",
        "google_compute_vpn_tunnel",
        "google_compute_route",
        "google_compute_address",
        "google_compute_global_address",
        "google_compute_router",
        "google_compute_interconnect_attachment",
        "google_compute_ha_vpn_gateway",
        "google_compute_forwarding_rule",
        "google_compute_network_firewall_policy",
        "google_compute_network_firewall_policy_rule",
        "cloudflare_static_route",
        "cloudflare_ipsec_tunnel",
    ]

    title: str


class SectionVisibleItemValueMcnResourcePreviewItem(BaseModel):
    item_type: str

    resource_preview: SectionVisibleItemValueMcnResourcePreviewItemResourcePreview


class SectionVisibleItemValueMcnListItemListMcnStringItem(BaseModel):
    item_type: str

    string: str


class SectionVisibleItemValueMcnListItemListMcnResourcePreviewItemResourcePreview(BaseModel):
    id: str

    cloud_type: Literal["AWS", "AZURE", "GOOGLE", "CLOUDFLARE"]

    detail: str

    name: str

    resource_type: Literal[
        "aws_customer_gateway",
        "aws_egress_only_internet_gateway",
        "aws_internet_gateway",
        "aws_instance",
        "aws_network_interface",
        "aws_route",
        "aws_route_table",
        "aws_route_table_association",
        "aws_subnet",
        "aws_vpc",
        "aws_vpc_ipv4_cidr_block_association",
        "aws_vpn_connection",
        "aws_vpn_connection_route",
        "aws_vpn_gateway",
        "aws_security_group",
        "aws_vpc_security_group_ingress_rule",
        "aws_vpc_security_group_egress_rule",
        "aws_ec2_managed_prefix_list",
        "aws_ec2_transit_gateway",
        "aws_ec2_transit_gateway_prefix_list_reference",
        "aws_ec2_transit_gateway_vpc_attachment",
        "azurerm_application_security_group",
        "azurerm_lb",
        "azurerm_lb_backend_address_pool",
        "azurerm_lb_nat_pool",
        "azurerm_lb_nat_rule",
        "azurerm_lb_rule",
        "azurerm_local_network_gateway",
        "azurerm_network_interface",
        "azurerm_network_interface_application_security_group_association",
        "azurerm_network_interface_backend_address_pool_association",
        "azurerm_network_interface_security_group_association",
        "azurerm_network_security_group",
        "azurerm_public_ip",
        "azurerm_route",
        "azurerm_route_table",
        "azurerm_subnet",
        "azurerm_subnet_route_table_association",
        "azurerm_virtual_machine",
        "azurerm_virtual_network_gateway_connection",
        "azurerm_virtual_network",
        "azurerm_virtual_network_gateway",
        "google_compute_network",
        "google_compute_subnetwork",
        "google_compute_vpn_gateway",
        "google_compute_vpn_tunnel",
        "google_compute_route",
        "google_compute_address",
        "google_compute_global_address",
        "google_compute_router",
        "google_compute_interconnect_attachment",
        "google_compute_ha_vpn_gateway",
        "google_compute_forwarding_rule",
        "google_compute_network_firewall_policy",
        "google_compute_network_firewall_policy_rule",
        "cloudflare_static_route",
        "cloudflare_ipsec_tunnel",
    ]

    title: str


class SectionVisibleItemValueMcnListItemListMcnResourcePreviewItem(BaseModel):
    item_type: str

    resource_preview: SectionVisibleItemValueMcnListItemListMcnResourcePreviewItemResourcePreview


SectionVisibleItemValueMcnListItemList: TypeAlias = Annotated[
    Union[
        SectionVisibleItemValueMcnListItemListMcnStringItem,
        SectionVisibleItemValueMcnListItemListMcnResourcePreviewItem,
    ],
    PropertyInfo(discriminator="item_type"),
]


class SectionVisibleItemValueMcnListItem(BaseModel):
    item_type: str

    rule_list: List[SectionVisibleItemValueMcnListItemList] = FieldInfo(alias="list")


SectionVisibleItemValue: TypeAlias = Annotated[
    Union[
        SectionVisibleItemValueMcnStringItem,
        SectionVisibleItemValueMcnYamlItem,
        SectionVisibleItemValueMcnYamlDiffItem,
        SectionVisibleItemValueMcnResourcePreviewItem,
        SectionVisibleItemValueMcnListItem,
    ],
    PropertyInfo(discriminator="item_type"),
]


class SectionVisibleItem(BaseModel):
    help_text: Optional[str] = FieldInfo(alias="helpText", default=None)

    name: Optional[str] = None

    value: Optional[SectionVisibleItemValue] = None


class Section(BaseModel):
    hidden_items: List[SectionHiddenItem]

    name: str

    visible_items: List[SectionVisibleItem]

    help_text: Optional[str] = None


class ManagedBy(BaseModel):
    id: str

    client_type: Literal["MAGIC_WAN_CLOUD_ONRAMP"]

    name: str


class ResourceGetResponse(BaseModel):
    id: str

    account_id: str

    cloud_type: Literal["AWS", "AZURE", "GOOGLE", "CLOUDFLARE"]

    config: Dict[str, object]

    deployment_provider: str

    managed: bool

    monthly_cost_estimate: MonthlyCostEstimate

    name: str

    native_id: str

    observations: Dict[str, Observations]

    provider_ids: List[str]

    provider_names_by_id: Dict[str, str]

    region: str

    resource_group: str

    resource_type: Literal[
        "aws_customer_gateway",
        "aws_egress_only_internet_gateway",
        "aws_internet_gateway",
        "aws_instance",
        "aws_network_interface",
        "aws_route",
        "aws_route_table",
        "aws_route_table_association",
        "aws_subnet",
        "aws_vpc",
        "aws_vpc_ipv4_cidr_block_association",
        "aws_vpn_connection",
        "aws_vpn_connection_route",
        "aws_vpn_gateway",
        "aws_security_group",
        "aws_vpc_security_group_ingress_rule",
        "aws_vpc_security_group_egress_rule",
        "aws_ec2_managed_prefix_list",
        "aws_ec2_transit_gateway",
        "aws_ec2_transit_gateway_prefix_list_reference",
        "aws_ec2_transit_gateway_vpc_attachment",
        "azurerm_application_security_group",
        "azurerm_lb",
        "azurerm_lb_backend_address_pool",
        "azurerm_lb_nat_pool",
        "azurerm_lb_nat_rule",
        "azurerm_lb_rule",
        "azurerm_local_network_gateway",
        "azurerm_network_interface",
        "azurerm_network_interface_application_security_group_association",
        "azurerm_network_interface_backend_address_pool_association",
        "azurerm_network_interface_security_group_association",
        "azurerm_network_security_group",
        "azurerm_public_ip",
        "azurerm_route",
        "azurerm_route_table",
        "azurerm_subnet",
        "azurerm_subnet_route_table_association",
        "azurerm_virtual_machine",
        "azurerm_virtual_network_gateway_connection",
        "azurerm_virtual_network",
        "azurerm_virtual_network_gateway",
        "google_compute_network",
        "google_compute_subnetwork",
        "google_compute_vpn_gateway",
        "google_compute_vpn_tunnel",
        "google_compute_route",
        "google_compute_address",
        "google_compute_global_address",
        "google_compute_router",
        "google_compute_interconnect_attachment",
        "google_compute_ha_vpn_gateway",
        "google_compute_forwarding_rule",
        "google_compute_network_firewall_policy",
        "google_compute_network_firewall_policy_rule",
        "cloudflare_static_route",
        "cloudflare_ipsec_tunnel",
    ]

    sections: List[Section]

    state: Dict[str, object]

    tags: Dict[str, str]

    updated_at: str

    url: str

    managed_by: Optional[List[ManagedBy]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .report import Report as Report
from .report_get_params import ReportGetParams as ReportGetParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["RedirectParam"]


class RedirectParam(TypedDict, total=False):
    """The definition of the redirect."""

    source_url: Required[str]

    target_url: Required[str]

    include_subdomains: bool

    preserve_path_suffix: bool

    preserve_query_string: bool

    status_code: Literal[301, 302, 307, 308]

    subpath_matching: bool



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from datetime import datetime
from typing_extensions import Literal

from ...._models import BaseModel

__all__ = ["UserUpdateResponse"]


class UserUpdateResponse(BaseModel):
    created_at: datetime

    description: str
    """The description of the label"""

    last_updated: datetime

    metadata: object
    """Metadata for the label"""

    name: str
    """The name of the label"""

    source: Literal["user", "managed"]
    """
    - `user` - label is owned by the user
    - `managed` - label is owned by cloudflare
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Iterable, cast
from typing_extensions import Literal

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ...._base_client import make_request_options
from ....types.workers.scripts import deployment_create_params
from ....types.workers.scripts.deployment import Deployment
from ....types.workers.scripts.deployment_list_response import DeploymentListResponse
from ....types.workers.scripts.deployment_delete_response import DeploymentDeleteResponse

__all__ = ["DeploymentsResource", "AsyncDeploymentsResource"]


class DeploymentsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> DeploymentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return DeploymentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DeploymentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return DeploymentsResourceWithStreamingResponse(self)

    def create(
        self,
        script_name: str,
        *,
        account_id: str,
        strategy: Literal["percentage"],
        versions: Iterable[deployment_create_params.Version],
        force: bool | Omit = omit,
        annotations: deployment_create_params.Annotations | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Deployment:
        """
        Deployments configure how
        [Worker Versions](https://developers.cloudflare.com/api/operations/worker-versions-list-versions)
        are deployed to traffic. A deployment can consist of one or two versions of a
        Worker.

        Args:
          account_id: Identifier.

          script_name: Name of the script, used in URLs and route configuration.

          force: If set to true, the deployment will be created even if normally blocked by
              something such rolling back to an older version when a secret has changed.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not script_name:
            raise ValueError(f"Expected a non-empty value for `script_name` but received {script_name!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/workers/scripts/{script_name}/deployments",
                account_id=account_id,
                script_name=script_name,
            ),
            body=maybe_transform(
                {
                    "strategy": strategy,
                    "versions": versions,
                    "annotations": annotations,
                },
                deployment_create_params.DeploymentCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"force": force}, deployment_create_params.DeploymentCreateParams),
                post_parser=ResultWrapper[Deployment]._unwrapper,
            ),
            cast_to=cast(Type[Deployment], ResultWrapper[Deployment]),
        )

    def list(
        self,
        script_name: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DeploymentListResponse:
        """List of Worker Deployments.

        The first deployment in the list is the latest
        deployment actively serving traffic.

        Args:
          account_id: Identifier.

          script_name: Name of the script, used in URLs and route configuration.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not script_name:
            raise ValueError(f"Expected a non-empty value for `script_name` but received {script_name!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/workers/scripts/{script_name}/deployments",
                account_id=account_id,
                script_name=script_name,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[DeploymentListResponse]._unwrapper,
            ),
            cast_to=cast(Type[DeploymentListResponse], ResultWrapper[DeploymentListResponse]),
        )

    def delete(
        self,
        deployment_id: str,
        *,
        account_id: str,
        script_name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DeploymentDeleteResponse:
        """Delete a Worker Deployment.

        The latest deployment, which is actively serving
        traffic, cannot be deleted. All other deployments can be deleted.

        Args:
          account_id: Identifier.

          script_name: Name of the script, used in URLs and route configuration.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not script_name:
            raise ValueError(f"Expected a non-empty value for `script_name` but received {script_name!r}")
        if not deployment_id:
            raise ValueError(f"Expected a non-empty value for `deployment_id` but received {deployment_id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/workers/scripts/{script_name}/deployments/{deployment_id}",
                account_id=account_id,
                script_name=script_name,
                deployment_id=deployment_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DeploymentDeleteResponse,
        )

    def get(
        self,
        deployment_id: str,
        *,
        account_id: str,
        script_name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Deployment:
        """
        Get information about a Worker Deployment.

        Args:
          account_id: Identifier.

          script_name: Name of the script, used in URLs and route configuration.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not script_name:
            raise ValueError(f"Expected a non-empty value for `script_name` but received {script_name!r}")
        if not deployment_id:
            raise ValueError(f"Expected a non-empty value for `deployment_id` but received {deployment_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/workers/scripts/{script_name}/deployments/{deployment_id}",
                account_id=account_id,
                script_name=script_name,
                deployment_id=deployment_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Deployment]._unwrapper,
            ),
            cast_to=cast(Type[Deployment], ResultWrapper[Deployment]),
        )


class AsyncDeploymentsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncDeploymentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDeploymentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDeploymentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncDeploymentsResourceWithStreamingResponse(self)

    async def create(
        self,
        script_name: str,
        *,
        account_id: str,
        strategy: Literal["percentage"],
        versions: Iterable[deployment_create_params.Version],
        force: bool | Omit = omit,
        annotations: deployment_create_params.Annotations | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Deployment:
        """
        Deployments configure how
        [Worker Versions](https://developers.cloudflare.com/api/operations/worker-versions-list-versions)
        are deployed to traffic. A deployment can consist of one or two versions of a
        Worker.

        Args:
          account_id: Identifier.

          script_name: Name of the script, used in URLs and route configuration.

          force: If set to true, the deployment will be created even if normally blocked by
              something such rolling back to an older version when a secret has changed.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not script_name:
            raise ValueError(f"Expected a non-empty value for `script_name` but received {script_name!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/workers/scripts/{script_name}/deployments",
                account_id=account_id,
                script_name=script_name,
            ),
            body=await async_maybe_transform(
                {
                    "strategy": strategy,
                    "versions": versions,
                    "annotations": annotations,
                },
                deployment_create_params.DeploymentCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"force": force}, deployment_create_params.DeploymentCreateParams),
                post_parser=ResultWrapper[Deployment]._unwrapper,
            ),
            cast_to=cast(Type[Deployment], ResultWrapper[Deployment]),
        )

    async def list(
        self,
        script_name: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DeploymentListResponse:
        """List of Worker Deployments.

        The first deployment in the list is the latest
        deployment actively serving traffic.

        Args:
          account_id: Identifier.

          script_name: Name of the script, used in URLs and route configuration.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not script_name:
            raise ValueError(f"Expected a non-empty value for `script_name` but received {script_name!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/workers/scripts/{script_name}/deployments",
                account_id=account_id,
                script_name=script_name,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[DeploymentListResponse]._unwrapper,
            ),
            cast_to=cast(Type[DeploymentListResponse], ResultWrapper[DeploymentListResponse]),
        )

    async def delete(
        self,
        deployment_id: str,
        *,
        account_id: str,
        script_name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DeploymentDeleteResponse:
        """Delete a Worker Deployment.

        The latest deployment, which is actively serving
        traffic, cannot be deleted. All other deployments can be deleted.

        Args:
          account_id: Identifier.

          script_name: Name of the script, used in URLs and route configuration.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not script_name:
            raise ValueError(f"Expected a non-empty value for `script_name` but received {script_name!r}")
        if not deployment_id:
            raise ValueError(f"Expected a non-empty value for `deployment_id` but received {deployment_id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/workers/scripts/{script_name}/deployments/{deployment_id}",
                account_id=account_id,
                script_name=script_name,
                deployment_id=deployment_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DeploymentDeleteResponse,
        )

    async def get(
        self,
        deployment_id: str,
        *,
        account_id: str,
        script_name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Deployment:
        """
        Get information about a Worker Deployment.

        Args:
          account_id: Identifier.

          script_name: Name of the script, used in URLs and route configuration.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not script_name:
            raise ValueError(f"Expected a non-empty value for `script_name` but received {script_name!r}")
        if not deployment_id:
            raise ValueError(f"Expected a non-empty value for `deployment_id` but received {deployment_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/workers/scripts/{script_name}/deployments/{deployment_id}",
                account_id=account_id,
                script_name=script_name,
                deployment_id=deployment_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Deployment]._unwrapper,
            ),
            cast_to=cast(Type[Deployment], ResultWrapper[Deployment]),
        )


class DeploymentsResourceWithRawResponse:
    def __init__(self, deployments: DeploymentsResource) -> None:
        self._deployments = deployments

        self.create = to_raw_response_wrapper(
            deployments.create,
        )
        self.list = to_raw_response_wrapper(
            deployments.list,
        )
        self.delete = to_raw_response_wrapper(
            deployments.delete,
        )
        self.get = to_raw_response_wrapper(
            deployments.get,
        )


class AsyncDeploymentsResourceWithRawResponse:
    def __init__(self, deployments: AsyncDeploymentsResource) -> None:
        self._deployments = deployments

        self.create = async_to_raw_response_wrapper(
            deployments.create,
        )
        self.list = async_to_raw_response_wrapper(
            deployments.list,
        )
        self.delete = async_to_raw_response_wrapper(
            deployments.delete,
        )
        self.get = async_to_raw_response_wrapper(
            deployments.get,
        )


class DeploymentsResourceWithStreamingResponse:
    def __init__(self, deployments: DeploymentsResource) -> None:
        self._deployments = deployments

        self.create = to_streamed_response_wrapper(
            deployments.create,
        )
        self.list = to_streamed_response_wrapper(
            deployments.list,
        )
        self.delete = to_streamed_response_wrapper(
            deployments.delete,
        )
        self.get = to_streamed_response_wrapper(
            deployments.get,
        )


class AsyncDeploymentsResourceWithStreamingResponse:
    def __init__(self, deployments: AsyncDeploymentsResource) -> None:
        self._deployments = deployments

        self.create = async_to_streamed_response_wrapper(
            deployments.create,
        )
        self.list = async_to_streamed_response_wrapper(
            deployments.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            deployments.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            deployments.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

from ..._types import SequenceNotStr

__all__ = ["ConfigCreateParams", "WARPDevice"]


class ConfigCreateParams(TypedDict, total=False):
    account_id: Required[str]

    default_sampling: Required[float]
    """Fallback sampling rate of flow messages being sent in packets per second.

    This should match the packet sampling rate configured on the router.
    """

    name: Required[str]
    """The account name."""

    router_ips: SequenceNotStr[str]

    warp_devices: Iterable[WARPDevice]


class WARPDevice(TypedDict, total=False):
    """Object representing a warp device with an ID and name."""

    id: Required[str]
    """Unique identifier for the warp device."""

    name: Required[str]
    """Name of the warp device."""

    router_ip: Required[str]
    """IPv4 CIDR of the router sourcing flow data associated with this warp device.

    Only /32 addresses are currently supported.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["CrowdstrikeInputParam"]


class CrowdstrikeInputParam(TypedDict, total=False):
    connection_id: Required[str]
    """Posture Integration ID."""

    last_seen: str
    """For more details on last seen, please refer to the Crowdstrike documentation."""

    operator: Literal["<", "<=", ">", ">=", "=="]
    """Operator."""

    os: str
    """Os Version."""

    overall: str
    """Overall."""

    sensor_config: str
    """SensorConfig."""

    state: Literal["online", "offline", "unknown"]
    """For more details on state, please refer to the Crowdstrike documentation."""

    version: str
    """Version."""

    version_operator: Annotated[Literal["<", "<=", ">", ">=", "=="], PropertyInfo(alias="versionOperator")]
    """Version Operator."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["FipsSettingsParam"]


class FipsSettingsParam(TypedDict, total=False):
    """Specify FIPS settings."""

    tls: bool
    """Enforce cipher suites and TLS versions compliant with FIPS 140-2."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["DomainJoinedInputParam"]


class DomainJoinedInputParam(TypedDict, total=False):
    operating_system: Required[Literal["windows"]]
    """Operating System."""

    domain: str
    """Domain."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .on_ramps import (
    OnRampsResource,
    AsyncOnRampsResource,
    OnRampsResourceWithRawResponse,
    AsyncOnRampsResourceWithRawResponse,
    OnRampsResourceWithStreamingResponse,
    AsyncOnRampsResourceWithStreamingResponse,
)
from .address_spaces import (
    AddressSpacesResource,
    AsyncAddressSpacesResource,
    AddressSpacesResourceWithRawResponse,
    AsyncAddressSpacesResourceWithRawResponse,
    AddressSpacesResourceWithStreamingResponse,
    AsyncAddressSpacesResourceWithStreamingResponse,
)

__all__ = [
    "AddressSpacesResource",
    "AsyncAddressSpacesResource",
    "AddressSpacesResourceWithRawResponse",
    "AsyncAddressSpacesResourceWithRawResponse",
    "AddressSpacesResourceWithStreamingResponse",
    "AsyncAddressSpacesResourceWithStreamingResponse",
    "OnRampsResource",
    "AsyncOnRampsResource",
    "OnRampsResourceWithRawResponse",
    "AsyncOnRampsResourceWithRawResponse",
    "OnRampsResourceWithStreamingResponse",
    "AsyncOnRampsResourceWithStreamingResponse",
]
